import * as x11 from 'x11';
import * as fs from 'fs-extra';
import * as os from 'os';
import * as pify from 'pify';

async function detectDbusSessionAddress() {
  // read machine uuid
  const uuid = await fs.readFile('/var/lib/dbus/machine-id', 'ascii');
  const hostname = os.hostname().split('-')[0];
  const { client, screen } = await pify(x11.createClient)();
  const username = process.env.USER || hostname || 'barco';
  const id = await pify(client.InternAtom.bind(client))(false, `_DBUS_SESSION_BUS_SELECTION_${username}_${uuid.trim()}`);
  let win;

  if (process.env.DEV_WITH_WINDOW_MANAGER) {
    win = screen[0].root;
  } else {
    win = await pify(client.GetSelectionOwner.bind(client))(id);
  }

  const propId = await pify(client.InternAtom.bind(client))(false, '_DBUS_SESSION_BUS_ADDRESS');
  const val = await pify(client.GetProperty.bind(client))(0, win, propId, 0, 0, 10000000);
  await pify(client.close.bind(client))();
  return val.data.toString();
}

export default detectDbusSessionAddress;

export { detectDbusSessionAddress };
