import * as xml2js from 'xml2js';
import * as pify from 'pify';

import DBusInterface from './interface';

function introspect(obj) {
  return obj.service.bus.invoke({
    destination: obj.service.name,
    path: obj.name,
    interface: 'org.freedesktop.DBus.Introspectable',
    member: 'Introspect',
  }).then(async (xml) => {
    // Uncomment line below to see dbus object properties...
    // console.log(xml);
    const parser = new xml2js.Parser({
      explicitArray: true,
    });
    const result = (await pify(parser.parseString.bind(parser))(xml)).node;

    if (!result.interface) {
      // if (result.node && result.node.length > 0 && result.node[0].$) {
      //    let subObj = Object.assign(obj, {});
      //    if (subObj.name.slice(-1) !== '/')
      //        subObj.name += '/';
      //    subObj.name += result.node[0].$.name;
      //    return introspect(subObj)
      // }
      throw new Error('No such interface found');
    }

    const proxy = {};
    const ifaces = result.interface;

    for (const iface of ifaces) {
      const ifaceName = iface.$.name;
      const currentIface = new DBusInterface(ifaceName, obj);
      proxy[ifaceName] = currentIface;

      if (iface.method) {
        for (const method of iface.method) {
          let signature = '';
          const { name } = method.$;

          if (method.arg) {
            for (let arg of method.arg) {
              arg = arg.$;
              if (arg.direction === 'in') { signature += arg.type; }
            }
          }

          // add method
          currentIface[name] = function generatedMethod(...args) {
            const msg = {
              destination: this.obj.service.name,
              path: this.obj.name,
              interface: this.name,
              member: name,
            };
            if (signature !== '') {
              msg.signature = signature;
              msg.body = args;
            }
            return this.bus.invoke(msg);
          };
        }
      }

      if (iface.property) {
        for (const property of iface.property) {
          const { name } = property.$;
          Object.defineProperty(currentIface, name, {
            enumerable: true,
            async get() {
              const val = (await this.get(name))[0];

              const signature = val[0];
              return signature.length === 1
                ? val[1][0]
                : val[1];
            },
            set(val) {
              this.set(name, [property.$.type, val]);
            },
          });
          currentIface[`set${name}`] = function generatedSetter(val) {
            return this.set(name, [property.$.type, val]);
          };
        }
      }

      if (iface.signal) {
        // for (const signal of iface.signal) {
        //   const { name } = signal.$;
        //   console.log('============    signal: ', name);
        // }
      }
    }

    return proxy;
  });
}

export default introspect;
