/*
eslint no-underscore-dangle: ["error", {
  "allow": ["_path", "_id"],
  "allowAfterThis": true,
  "allowAfterSuper": true,
}]
*/
/*
eslint-disable no-await-in-loop, no-plusplus
*/
import * as _ from 'lodash';
import CacheManager from '../../../services/cacheManager';

import { sourceAddedBy } from '../../../services/ddp/constants';

class PaCacheManager extends CacheManager {
  constructor(paClient) {
    super(['sinks', 'sources', 'playbackStreams', 'clients', 'modules', 'cards']);
    this._paClient = paClient;
  }

  /**
   * Fetch + Fill the cache
   */
  async warm(autoRetryTillSucceeded = false) {
    await this.updateCards(autoRetryTillSucceeded);
    await this.updateSources(autoRetryTillSucceeded);
    await this.updateModules(autoRetryTillSucceeded);
    await this.updateSinks(autoRetryTillSucceeded);
    await this.updatePlaybackStreams(autoRetryTillSucceeded);
  }

  // async updateAudioDevices() {
  //   this._log('verbose', `Syncing audio devices`)
  //   const audioDevices = await this._paClient.getAudioDevices({
  //     includePath: true,
  //     includeIface: true
  //   })
  //   if (audioDevices) {
  //     console.log("+++++++++++++++++++++++++++++++++++++++++=");
  //     this._log('verbose', JSON.stringify(this.clean(audioDevices), null, 2));
  //     console.log("+++++++++++++++++++++++++++++++++++++++++=");
  //     this.setAudioDevices(audioDevices.audioDevices)
  //   }
  // }

  // setAudioDevices(audioDevices) {
  //   return this._cache({
  //     entity: 'audioDevices',
  //     items: audioDevices,
  //     add: false
  //   });
  // }

  // resetAudioDevices() {
  //   this._log('verbose', `Resetting audio devices`)
  //   return this._reset(['audioDevices']);
  // }

  // addAudioDevices(audioDevices) {
  //   return this._cache({
  //     entity: 'audioDevices',
  //     items: audioDevices,
  //     add: true
  //   });
  // }

  // getLocalCombinedSink(){
  //   return this._data.$arrays.sinks.find(sink => true) //XXX
  // }

  // getRemoteCombinedSinks(){
  //   return this._data.$arrays.sinks.find(sink => true) //XXX
  // }

  // getCombinedSink(){
  //   return this._data.$arrays.sinks.find(sink => true) //XXX
  // }

  // getVirtualSinks(){
  //   return this._data.$arrays.sinks.filter(sink => true)//XXX
  // }

  async updateCards(autoRetryTillSucceeded = false) {
    let attempt = 0;
    do {
      this._log('verbose', `Syncing audio cards attempt ${++attempt}`);
      try {
        const cards = await this._paClient.getCards({
          includePath: true,
          includeIface: true,
        });
        if (cards) {
          this._log('verbose', 'Number of audio cards found = ', cards.cards.length);
          this._log('verbose', JSON.stringify(this.clean(cards)));
          this.setCards(cards.cards);
        }
        break;
      } catch (error) {
        this._log('error', 'Error while syncing audio cards', error);
      }
    } while (autoRetryTillSucceeded && this.wantsToBeActive);
  }

  setCards(cards) {
    return this._cache({
      entity: 'cards',
      items: cards,
      add: false,
    });
  }

  resetCards() {
    this._log('verbose', 'Resetting cards');
    return this._reset(['cards']);
  }

  addCards(cards) {
    return this._cache({
      entity: 'cards',
      items: cards,
      add: true,
    });
  }

  removeCards(cards) {
    this._log('verbose', `Removing audio cards: ${JSON.stringify(this.clean(cards))}`);
    cards.forEach(card => this.remove('cards', card));
  }

  findCardByPath(path) {
    return this._data.$arrays.cards.find(card => card._path === path);
  }

  findCardByName(name) {
    return this._data.$arrays.cards.find(card => card.device === name);
  }

  findCardById(id) {
    return this._data.$arrays.cards.find(card => card.id === id);
  }

  getHardwareSinks() {
    return this._data.$arrays.sinks.filter(sink => sink.activePort !== null);
  }

  async updateSinks(autoRetryTillSucceeded = false) {
    let attempt = 0;
    do {
      this._log('verbose', `Syncing audio sinks attempt ${++attempt}`);
      try {
        const sinks = await this._paClient.getSinks({
          includePath: true,
          includeIface: true,
        });
        if (sinks) {
          this._log('verbose', 'Number of audio sinks found = ', sinks.sinks.length);
          this._log('verbose', JSON.stringify(this.clean(sinks)));
          this.setSinks(sinks.sinks);
        }
        break;
      } catch (error) {
        this._log('error', 'Error while syncing audio sinks', error);
      }
    } while (autoRetryTillSucceeded && this.wantsToBeActive);
  }

  setSinks(sinks) {
    return this._cache({
      entity: 'sinks',
      items: sinks,
      add: false,
    });
  }

  resetSinks() {
    this._log('verbose', 'Resetting sinks');
    return this._reset(['sinks']);
  }

  addSinks(sinks) {
    return this._cache({
      entity: 'sinks',
      items: sinks,
      add: true,
    });
  }

  removeSinks(sinks) {
    this._log('verbose', `Removing audio sinks: ${JSON.stringify(this.clean(sinks))}`);
    sinks.forEach(sink => this.remove('sinks', sink));
  }

  findSinkByPath(path) {
    return this._data.$arrays.sinks.find(sink => sink._path === path);
  }

  findSinkById(id) {
    return this._data.$arrays.sinks.find(sink => sink.id === id);
  }

  findNullSink() {
    return this._data.$arrays.sinks.find(sink => sink.id === 'nullsink');
  }

  findSourceById(id) {
    return this._data.$arrays.sources.find(source => source.id === id);
  }

  findNullSource() {
    return this._data.$arrays.sources.find(source => source.id === 'nullsink.monitor');
  }

  findTileMixSource() {
    return this._data.$arrays.sources.find(source => source.id === 'tile_mix.monitor');
  }

  async updateModules(autoRetryTillSucceeded = false) {
    let attempt = 0;
    do {
      this._log('verbose', `Syncing audio modules attempt ${++attempt}`);
      try {
        const modules = await this._paClient.getModules({
          includePath: true,
          includeIface: true,
        });
        if (modules) {
          this._log('verbose', 'Number of audio modules found = ', modules.modules.length);
          this._log('verbose', JSON.stringify(this.clean(modules)));
          this.setModules(modules.modules);
        }
        break;
      } catch (error) {
        this._log('error', 'Error while syncing audio modules', error);
      }
    } while (autoRetryTillSucceeded && this.wantsToBeActive);
  }

  logModules() {
    this._log('verbose', JSON.stringify(this.clean(this._data.modules), null, 2));
  }

  tunnelSinkToIPAddressExists(ipAddress) {
    const module = this._data.$arrays.modules.find(mod => (
      (mod.name === 'module-tunnel-sink-new')
      && (mod.hasOwnProperty('arguments'))
      && (mod.arguments.hasOwnProperty('server'))
      && (mod.arguments.server === ipAddress)
    ));
    return (!!module);
  }

  setModules(modules) {
    return this._cache({
      entity: 'modules',
      items: modules,
      add: false,
    });
  }

  findModulesByName(name) {
    if (!name) return [];
    return this._data.$arrays.modules.filter(module => (module.name === name));
  }

  findModulesLoadedByUDevDetectModule() {
    return this._data.$arrays.modules.filter(module => _.isObject(module.arguments) && (module.arguments.card_properties === 'module-udev-detect.discovered=1'));
  }

  findModulesBySinkName(name) {
    if (!name) return [];
    return this._data.$arrays.modules.filter(module => (
      _.isObject(module.arguments) && (module.arguments.sink_name === name || module.arguments.sink === name)
    ));
  }

  findModulesForWhichSinkNameIncludes(name) {
    if (!name) return [];
    return this._data.$arrays.modules.filter(module => (
      _.isObject(module.arguments)
      && ((module.arguments.sink_name && module.arguments.sink_name.includes(name))
      || (module.arguments.sink && module.arguments.sink.includes(name)))
    ));
  }

  getAllModules() {
    return this._data.$arrays.modules;
  }

  findModuleByPath(path) {
    // console.log('FINDING MODULE BY PATH, PATH =', path, 'NUMBER OF MODULES', this._data.$arrays.modules.length);
    // return this._data.$arrays.modules.find(module => module._path === path);
    return this._data.$arrays.modules.find(module => (module._path === path));
  }

  async updateSources(autoRetryTillSucceeded = false) {
    let attempt = 0;
    do {
      this._log('verbose', `Syncing audio sources attempt ${++attempt}`);
      try {
        const sources = await this._paClient.getSources({
          includePath: true,
          includeIface: true,
        });
        if (sources) {
          this._log('verbose', 'Number of audio sources found = ', sources.sources.length);
          this._log('verbose', JSON.stringify(this.clean(sources)));
          this.setSources(sources.sources);
        }
        break;
      } catch (error) {
        this._log('error', 'Error while syncing audio sources', error);
      }
    } while (autoRetryTillSucceeded && this.wantsToBeActive);
  }

  setSources(sources) {
    return this._cache({
      entity: 'sources',
      items: sources,
      add: false,
    });
  }

  resetSources() {
    this._log('verbose', 'Resetting sources');
    return this._reset(['sources']);
  }

  addSources(sources) {
    return this._cache({
      entity: 'sources',
      items: sources,
      add: true,
    });
  }

  removeSources(sources) {
    this._log('verbose', `Removing audio sources: ${JSON.stringify(this.clean(sources))}`);
    sources.forEach(source => this.remove('sources', source));
  }

  getHardwareSources() {
    return this._data.$arrays.sources.filter(source => source.activePort !== null);
  }

  findSourceByPath(path) {
    return this._data.$arrays.source.find(source => source._path === path);
  }

  handlePlaybackStream(stream) {
    // We only want to see streams in the cache that are coming from
    // ICGPUCompositor, edu_proxy (electron), barco-edu-rtspserver, gst-launch-1.0
    const processesToHandle = ['ICGPUCompositor', 'barco-edu-rtspserver', 'electron', 'edu_proxy', 'gst-launch-1.0'];
    const keepStream = processesToHandle.includes(stream.process);
    if (!keepStream) {
      this._log('verbose', `Not retaining stream from process '${stream.process}' with id '${stream.id}'. Not adding it to the stream cache.`);
    } else {
      this._log('verbose', `Adding stream from process '${stream.process}' with id '${stream.id}' to the stream cache.`);
    }
    return keepStream;
  }

  async updatePlaybackStreams(autoRetryTillSucceeded = false) {
    let attempt = 0;
    do {
      this._log('verbose', `Update audio playback streams attempt ${++attempt}`);
      try {
        const streams = await this._paClient.getPlaybackStreams({
          includePath: true,
          includeIface: true,
        });
        if (streams) {
          this._log('verbose', 'Number of audio playback streams found = ', streams.playbackStreams.length);
          this.setPlaybackStreams(streams.playbackStreams);
        }
        break;
      } catch (error) {
        this._log('error', 'Error while audio updating playback streams', error);
      }
    } while (autoRetryTillSucceeded && this.wantsToBeActive);
  }

  setPlaybackStreams(playbackStreams) {
    return this._cache({
      entity: 'playbackStreams',
      items: playbackStreams.filter(this.handlePlaybackStream.bind(this)),
      add: false,
    });
  }

  resetPlaybackStreams() {
    this._log('verbose', 'Resetting playbackStreams');
    return this._reset(['playbackStreams']);
  }

  addPlaybackStreams(playbackStreams) {
    this._log('verbose', `Adding audio playbackStreams: ${JSON.stringify(this.clean(playbackStreams))}`);
    return this._cache({
      entity: 'playbackStreams',
      items: playbackStreams.filter(this.handlePlaybackStream.bind(this)),
      add: true,
    });
  }

  removePlaybackStreams(playbackStreams) {
    this._log('verbose', `Removing audio playbackStreams: ${JSON.stringify(this.clean(playbackStreams))}`);
    playbackStreams.forEach(playbackStream => this.remove('playbackStreams', playbackStream.id));
  }

  findPlaybackStreamByPath(path) {
    return this._data.$arrays.playbackStreams.find(stream => stream._path === path);
  }

  // async updateClients() {
  //   this._log('verbose', `Update clients`)
  //   const clients = await this._paClient.getClients({
  //     includePath: true,
  //     includeIface: true
  //   })
  //   if (clients) {
  //     this.setClients(clients.clients);
  //   }

  // }

  // setClients(clients) {
  //   return this._cache({
  //     entity: 'clients',
  //     items: clients,
  //     add: false
  //   });
  // }

  // resetClients() {
  //   this._log('verbose', `Resetting clients`)
  //   return this._reset(['clients']);
  // }

  // findClientByPath(path) {
  //   return this._data.$arrays.clients.find(client => client._path === path);
  // }

  _cache({
    entity,
    items,
    add,
    identifier = item => (item.hasOwnProperty('_path') ? item._path : item.id), // only _path is unique (eg. playback streams)
  }) {
    this._log('verbose', `Caching ${entity}: ${JSON.stringify(this.clean(items))} (add:${add})`);
    if (!add) {
      this.reset([entity]);
    }
    items.forEach(item => this.add(entity, identifier(item), item));
  }

  findMeteorInput(input) {
    if (!input || !input.id) {
      return undefined;
    }
    return this._data.$arrays.sources.find(source => source.id === input.id);
  }

  findMeteorOutput(output) {
    if (!output || !output.id) {
      return undefined;
    }
    return this._data.$arrays.sinks.find(sink => sink.id === output.id);
  }

  findMeteorSource(source) {
    if (!source || !source._id) {
      return undefined;
    }
    return this._data.$arrays.playbackStreams.find(stream => stream.id === this._sourceToStreamName(source));
  }

  _sourceToStreamName(source) {
    if (source.hasOwnProperty('channelId')) {
      return `${source.deviceId}_${source.channelId}`;
    } else if (source.hasOwnProperty('origin') && source.addedBy[0].trigger === sourceAddedBy.DEVICE) {
      return source.origin.cinergyId;
    }
    return `BLUE_${source._id}`;
  }

  clean(object) {
    if (Array.isArray(object)) {
      return object.map(el => this.clean(el));
    } else if (_.isObject(object)) {
      return Object.keys(object).reduce((acc, el) => {
        if ((el.substr(0, 1) !== '_') || (el.substr(0, 5) === '_path')) {
          acc[el] = this.clean(object[el]);
        }
        return acc;
      }, {});
    }
    return object;
  }

  /**
   * Clean-up
   */
  _onDestroy() {
    delete this._paClient;
    return super._onDestroy();
  }
}

export default PaCacheManager;
