import CacheManager from '../../../services/cacheManager';

/**
 * Cache manager for the video 4 linux camera service
 * Contains 1 entity: 'cameras'
 */
class V4LCacheManager extends CacheManager {
  constructor() {
    super(['cameras']);
  }

  /**
   * Replaces the current cache with the new camera list
   * @param {*} cameras
   */
  setCameras(cameras) {
    return this._cache({
      entity: 'cameras',
      items: cameras,
      add: false,
    });
  }

  /**
   * Cleans/wipes the current cache
   */
  resetCameras() {
    this._log('verbose', 'Resetting cameras');
    return this._reset(['cameras']);
  }

  /**
   * Add cameras to the current cache
   * @param {*} cameras
   */
  addCameras(cameras) {
    return this._cache({
      entity: 'cameras',
      items: cameras,
      add: true,
    });
  }

  _cache({
    entity,
    items,
    add,
    identifier = item => item.id,
  }) {
    this._log('verbose', `Caching ${entity}: ${JSON.stringify(items)} (add:${add})`);
    if (!add) {
      this.reset([entity]);
    }
    items.forEach(item => this.add(entity, identifier(item), item));
  }
}

export default V4LCacheManager;
