import Server from '../../../services/server';
import serviceRegistry from '../../../services/serviceRegistry';

/**
 * The AuthBridge allows a cinergy CEF window to request a token
 * Since only authorized pages from the server can request from localhost, this is secure.
 */
class AuthBridge extends Server {
  constructor(options) {
    super({
      port: 8070,
      host: '127.0.0.1',
      ...options,
    });

    this.app.get('/device-token', this._requestDeviceToken.bind(this));
  }

  get serviceName() {
    return 'Cinergy/AuthBridge';
  }

  _requestDeviceToken(req, res) {
    this._log('verbose', 'Requesting device token');
    const token = serviceRegistry.get('configManager').getSetting('credentials.resume.token');
    res.end(token);
  }
}

export default AuthBridge;
