import MeteorCacheManager from '../../../services/ddp/meteorCacheManager';
import utils from './utils';
/**
 * Extends the Meteor cache with some helper functions
 */
class CinergyMeteorCacheManager extends MeteorCacheManager {
  /**
   * Given a cng composition object, find the corresponding Meteor composition object.
   * Compositions are mapped by storing the mongo id in the title of the cng composition object
   * @param cngComp
   * @returns {*}
   */
  findCngComposition(cngComp) {
    if (!cngComp) { return undefined; }
    return this._data.compositions[cngComp.title];
  }

  /**
   * Given a cng sink object, find the corresponding Meteor sink object.
   * Sinks are mapped by storing the mongo id in the device id of the cng sink object
   * @param cngSink
   * @returns {*}
   */
  findCngSink(cngSink) {
    if (!cngSink) { return undefined; }
    return this._data.sinks[cngSink.deviceId];
  }

  /**
   * Given a cng layout object, find the corresponding Meteor layout object.
   * layouts are mapped by storing the mongo id in the title of the layout
   * Since CNG layouts can not be changed, we also store a version number in the title.
   * This needs to be stripped of by the `parseLayoutTitle` function
   *
   * @param cngLayout
   * @returns {*}
   */
  findCngLayout(cngLayout) {
    if (!cngLayout || !cngLayout.title) { return undefined; }
    const parsed = utils.parseLayoutTitle(cngLayout.title);
    return this._data.layouts[parsed._id];
  }

  /**
   * Given a cng source object, find the corresponding Meteor source object.
   * Sources are mapped by a key which looks like: channelId_deviceId
   *
   * @param cngSource
   * @returns {*}
   */
  findCngSource(cngSource) {
    if (!cngSource) { return undefined; }
    const key = utils.sourceToKey(cngSource);
    return this._data.$arrays.sources.find(source => key === utils.sourceToKey(source));
  }
}

export default CinergyMeteorCacheManager;
