/**
 * Helper functions for the bridge and caches
 */

import { sourceAddedBy, state as deviceState } from '../../../services/ddp/constants';

export default {

  calculateDisplaysBB(displays = []) {
    let bb = [0, 0, 0, 0];

    if (displays.length) {
      bb = displays.reduce((prev, cur) => [
        Math.min(prev[0], cur.x),
        Math.min(prev[1], cur.y),
        Math.max(prev[2], (cur.x + cur.activeMode.width)),
        Math.max(prev[3], (cur.y + cur.activeMode.height)),
      ], [Number.POSITIVE_INFINITY, Number.POSITIVE_INFINITY, Number.NEGATIVE_INFINITY, Number.NEGATIVE_INFINITY]);
    }

    let [x, y, width, height] = bb;

    width -= x;
    height -= y;

    return {
      x,
      y,
      width,
      height,
    };
  },

  getState(object, state) {
    return (object.states || []).find(s => s.state === state) || {};
  },

  getFallbackSettings(object) {
    const defaults = this.getState(object, deviceState.DEFAULT);
    return { ...defaults, ...this.getState(object, deviceState.FALLBACK) };
  },

  compositorIdForComposition(composition) {
    return `screen${composition.index ? composition.index : ''}`;
  },

  /**
   * Helper method to compare meteor sources with cng sources
   * @param source
   * @returns {*}
   * @private
   */
  sourceToKey(source) {
    if (source.hasOwnProperty('channelId')) {
      return `${source.deviceId}_${source.channelId}`;
    } else if (source.hasOwnProperty('origin') && source.addedBy[0].trigger === sourceAddedBy.DEVICE) {
      return source.origin.cinergyId;
    }
    return `BLUE_${source._id}`;
  },

  keyToSource(key) {
    const [deviceId, channelId] = key.split('_');
    return {
      deviceId,
      channelId,
    };
  },

  /**
   * Extract mongo id and version number from a layout title
   * In order to map CNG layouts with Meteor layouts, we store the mongo id in the layout title.
   * Since layouts can not be overridden, we also add a version number to it.
   * This method extracts these two parts
   *
   * @param title
   * @returns {*}
   * @private
   */
  parseLayoutTitle(title) {
    const parts = title.split('_');
    if (parts.length === 2) {
      return {
        _id: parts[0],
        index: Number.parseInt(parts[1], 10),
      };
    }
    return {
      _id: parts[0],
      index: 0,
    };
  },

};
