/**
 * (C) 2015 Copyright BARCO NV
 * All rights reserved. Reproduction in whole or in part is
 * prohibited without the written consent of the copyright owner.
 *
 * Part of Barco WeConnect
 */
import * as x11 from 'x11';
import { promisify } from 'util';

import DisplayService from './service';

const createClient = promisify(x11.createClient);

/**
 * Display service based on node x11 implementation
 * Receives updates via x11 events
 *
 * NOTE: WE CURRENTLY USE THE UDEV SERVICE INSTEAD OF THIS ONE
 */
class X11DisplayService extends DisplayService {
  async _onStart() {
    this._display = await createClient();
    const X = this._display.client;
    const XRequire = promisify(X.require.bind(X));
    const XRandr = await XRequire('randr');
    const { root } = this._display.screen[0];

    X.on('error', (error) => {
      this._log('error', 'Error from x11', error);
    });

    X.on('event', (ev, ...args) => {
      if (ev.name === 'RRScreenChangeNotify') {
        this._log('info', `Event received ${ev.name}`);
        this.emit('displaysChanged', args);
      }
    });

    XRandr.SelectInput(root, XRandr.NotifyMask.ScreenChange);
    return super._onStart();
  }

  _onStop() {
    this._display.client.removeAllListeners();
    this._display.client.close();
    delete this._display;
    return super._onStop();
  }
}

export default X11DisplayService;
