import Bridge from '../bridge';
import HealthChecker from './service';
import { state as serviceState } from '../../../services/service';

const checkOnStartSec = 60;
const defaultSchedule = process.env.HEALTH_CHECK_SCHEDULE || '45 * * * *'; // 45'th minute of each hour (try to solve issues before lecture starts)

/**
 * The health bridge is the glue between blue and the device health service.
 */
class HealthBridge extends Bridge {
  constructor(options) {
    super(options);
    this._healthChecker = new HealthChecker(
      {
        id: this._device._id,
        meetingActive: !!this._device.meetingId,
        schedule: this.schedule,
        checkOnStartSec,
      },
    );
  }

  async _onStart() {
    await super._onStart();
    this._healthChecker.schedule = this.schedule;
    return this._healthChecker.start({ cancelToken: this.getCancelToken(serviceState.STARTED) });
  }

  async _onStop() {
    await super._onStop();
    return this._healthChecker.stop();
  }

  _onThisDeviceChanged(device, oldFields, clearedFields, newFields) {
    if (newFields.hasOwnProperty('healthCheckSchedule')) {
      this._log('verbose', 'Setting new device health check schedule');
      this._healthChecker.schedule = newFields.healthCheckSchedule;
    }
    if (newFields.hasOwnProperty('meetingId')) {
      this._healthChecker.meetingActive = !!newFields.meetingId;
    }
  }

  get schedule() {
    return this._device.healthCheckSchedule || defaultSchedule;
  }

  async _onDestroy() {
    await this._healthChecker.destroy();
    delete this._healthChecker;
    await super._onDestroy();
  }
}

export default HealthBridge;
