import Bridge from '../bridge';
import serviceRegistry from '../../../services/serviceRegistry';
import { commands } from '../../../services/ddp/constants';

/**
 * The identify bridge handles the identification messages
 * It glues blue with the messenger service
 */
class IdentifyBridge extends Bridge {
  get supportedRPCCommands() {
    return [commands.IDENTIFY];
  }

  /**
   * Act on incoming client RPCs
   * RPC's are useful for one-time actions that are not part of the application state.
   *
   * @param rpc
   * @returns {*}
   * @private
   */
  _onRpcAdded(rpc) {
    this._log('verbose', `Handling rpc ${JSON.stringify(rpc, null, ' ')}`, { serviceLog: true });
    switch (rpc.command) {
    case commands.IDENTIFY:
      this.identify();
      break;
    default:
    }
  }

  unidentify() {
    if (this._message) {
      this._message.hide();
      delete this._message;
    }
  }

  identify() {
    this.unidentify();
    const messenger = serviceRegistry.get('messenger');
    messenger.identify({ ip: this._device.privateIp, mac: this._device.mac });
    this._message = messenger.show({
      text: this._device.title,
      time: 25,
      flash: true,
    });
    this._message.then(() => {
      // when this resolves, the message is already hidden
      delete this._message;
    });
    return this._message;
  }

  async _onStop() {
    await super._onStop();
    await this.unidentify();
  }
}

export default IdentifyBridge;
