import KalturaUploader from './kalturaUploader';
import NASUploader from './NASUploader';
import PanoptoUploader from './panoptoUploader';
import { waitMaximum } from '../../../../utils/utils';

const KALTURA = 'kaltura';
const SCP = 'scp';
const PANOPTO = 'panopto';
const NONE = 'none';

const uploaderType = {
  [SCP]: NASUploader,
  [KALTURA]: KalturaUploader,
  [PANOPTO]: PanoptoUploader,
};

function createUploader(type, args) {
  if (!uploaderType.hasOwnProperty(type)) {
    throw new Error(`Uploader type ${type} not known in the system`);
  }

  if (args.backupSettings && args.backupSettings.uploadType !== NONE) {
    // Create backupUploader and add it to constructor args
    args.backupUploader = createUploader(
      args.backupSettings.uploadType, {
        filePath: args.filePath,
        metadata: args.metadata,
        settings: args.backupSettings,
        backup: true,
      },
    );
  }

  return new uploaderType[type](args);
}

async function validateUploaderOptions(options) {
  const Uploader = uploaderType[options.uploadType];
  if (!Uploader) {
    throw new Error('Unknown upload type');
  }

  return waitMaximum(Uploader.validate(options), 120 * 1000);
}
export default createUploader;
export { createUploader, uploaderType, validateUploaderOptions };
