import Stream from './stream';
import { Random } from '../../../utils/random';

/**
 * Audio-only RTSP stream used to pull audio from another device to this device
 */

// Digital audio mix: Increased default audio buffer to 50ms
// to avoid audio hiccups. Lipsync is still acceptable.
const AS_LATENCY = Number.parseInt(process.env.AS_LATENCY, 10) || 50;

class IncomingAudioStream extends Stream {
  constructor({
    path,
    pulseSinkName,
    username,
    password,
  }) {
    super({
      id: Random.id(),
      unsetXdgRuntimeDir: false,
    });
    this.path = path;
    this.pulseSinkName = pulseSinkName;
    this.username = username;
    this.password = password;
    this._log('info', `Creating audio stream for path ${path}`);
  }

  get command() {
    return 'gst-launch-1.0';
  }

  get streamType() {
    return 'AudioFromRtsp';
  }

  get args() {
    const auth = this.username ? ` user-id=${this.username} user-pw=${this.password}` : '';
    const args = `rtspsrc location=${this.path}${auth} latency=${AS_LATENCY} ! rtpL16depay ! pulsesink client-name=${this.pulseSinkName}`;
    // For manual testing use:
    // gst-launch-1.0 rtspsrc location=rtsp://192.168.0.101:8554/audio latency=1 ! rtpL16depay ! pulsesink  OR
    // gst-launch-1.0 rtspsrc location=rtsp://192.168.0.101:8554/audio latency=1 ! decodebin ! pulsesink
    return args;
  }

  equals(stream) {
    return stream instanceof IncomingAudioStream
      && (this.path === stream.path)
      && (this.pulseSinkName === stream.pulseSinkName)
      && (this.username === stream.username)
      && (this.password === stream.password);
  }
}

export default IncomingAudioStream;
