import Stream from './stream';

/**
 * MJPEG stream
 */
class MJPEGStream extends Stream {
  constructor({
    port, path, width, height, framerate, params,
  }) {
    super({ id: path });

    this.path = path;
    this.port = port;
    this.width = width;
    this.height = height;
    this.framerate = framerate;
    this.params = params;

    this._log('info', `Creating jpeg stream for ${path} at 0.0.0.0:${port}`);
  }

  get command() {
    return 'gst-launch-1.0';
  }

  get streamType() {
    return 'MJPEG';
  }

  get args() {
    const jpegpath = `${this.path}.jpeg`;
    const jpegquality = 65;
    // The pipeline below used to split in an mjpeg and a jpeg part. Since the
    // mjpeg part no longer in use since chrome 86, that part has been removed.
    let args = `shmsrc is-live=true socket-path=${this.path} do-timestamp=true ! video/x-raw, width=${this.width}, height=${this.height}, framerate=${this.framerate.denominator}/${this.framerate.numerator}, format=I420, pixel-aspect-ratio=1/1 ! `;
    // args += `videorate max-rate=${this.framerate.denominator / this.framerate.numerator} ! tee name=t ! queue max-size-buffers=1 max-size-bytes=0 leaky=2 ! jpegenc ! multipartmux ! tcpserversink host=0.0.0.0 port=${this.port} sync=false t. ! videorate max-rate=1 ! queue max-size-buffers=1 max-size-bytes=0 leaky=2 ! jpegenc quality=${jpegquality} ! multifilesink location=${jpegpath} next-file=buffer max-files=1`;
    args += `videorate max-rate=1 ! queue max-size-buffers=1 max-size-bytes=0 leaky=2 ! jpegenc quality=${jpegquality} ! multifilesink location=${jpegpath} next-file=buffer max-files=1`;

    // Replace by line below for debugging...
    // args += "videoconvert ! xvimagesink max-lateness=-1";
    if (this.params) {
      args += ` ${this.params}`;
    }
    return args;
  }

  equals(stream) {
    return stream instanceof MJPEGStream
      && this.width === stream.width
      && this.height === stream.height
      && this.framerate.numerator === stream.framerate.numerator
      && this.framerate.denominator === stream.framerate.denominator
      && this.path === stream.path
      && this.port === stream.port
      && this.params === stream.params;
  }
}

export default MJPEGStream;
