import Stream from './stream';
import { Random } from '../../../utils/random';

/**
 * RTSP server that serves an audio-only stream (used to send audio from one device to another)
 */
class RTSPAudioStream extends Stream {
  constructor({
    port,
    url,
    params,
    paSrcDevice,
    username,
    password,
  }) {
    super({ id: Random.id() });

    this.port = port;
    this.url = url;
    this.params = params;
    this.paSrcDevice = paSrcDevice;
    this.username = username;
    this.password = password;
  }

  get command() {
    return '/usr/lib/barco-edu-rtspserver/barco-edu-rtspserver';
  }

  get streamType() {
    return 'RTSP_AUDIO';
  }

  get args() {
    let args = `-p ${this.port} -u /audio -s -1 -d `;

    if (this.paSrcDevice) {
      args += this.paSrcDevice;
    } else {
      args += '-1 -n';
    }

    if (this.username) {
      args += ` --username ${this.username}`;
    }

    if (this.password) {
      args += ` --password ${this.password}`;
    }

    if (this.params) {
      args += ` ${this.params}`;
    }
    return args;
  }

  equals(stream) {
    return stream instanceof RTSPAudioStream
      && (this.port === stream.port)
      && (this.url === stream.url)
      && (this.params === stream.params)
      && (this.paSrcDevice === stream.paSrcDevice)
      && (this.username === stream.username)
      && (this.password === stream.password);
  }

  _onStart() {
    this._log('info', `Streaming audio only RTSP at 0.0.0.0:${this.port}/${this.url}`);
    return super._onStart();
  }
}

export default RTSPAudioStream;
