import RTSPStream from './rtspStream';

/**
 * RTSP stream from shared memory
 */
class RTSPScreenStream extends RTSPStream {
  constructor({
    source,
    width,
    height,
    framerate,
    ...options
  } = {}) {
    options.params = `-v "video/x-raw,framerate=${framerate.denominator}/${framerate.numerator}`;
    if (source.width !== width || source.height !== height) {
      // add scaler.
      // add-borders=true => preserve aspect ratio
      // Standard will use nearest-neighbor (fast).
      // See 'enum GstVideoScaleMethod' at https://gstreamer.freedesktop.org/data/doc/gstreamer/head/gst-plugins-base-plugins/html/gst-plugins-base-plugins-videoscale.html
      // for alternatives. Bicubic + sharper is best for scaling down. Bicubic without sharpner is best for scaling up, but both pretty slow
      options.params += ' ! videoscale ! video/x-raw';
    }
    options.params += `,width=${width},height=${height},format=BGRx"`;

    options.path = `/dev/screen?startx=${source.x}&starty=${source.y}&endx=${source.x + source.width - 1}&endy=${source.y + source.height - 1}`;
    super(options);
  }
}

export default RTSPScreenStream;
