/*
eslint-disable
*/
import { assert } from 'chai';
import { generateListOfDevicesToPullAudioFrom } from './streamingBridge';


describe('Test digital audio mixing functionality', () => {
  let dummyLogger;
  const devicesHybridLecture = [
    {
      "_id" : "M3NmAcvCALHY7sZv3",
      "meetingId" : "A7FjKihaate5LRTKv",
      "audioSinkConfig" : [ 
          {
              "id" : "alsa_output.pci-0000_00_1b.0.analog-stereo",
              "activePort" : {
                  "id" : "analog-output-speaker",
                  "title" : "Speakers"
              },
              "device" : "Built-in Audio Analog Stereo",
              "role" : "idle"
          }, 
          {
              "id" : "alsa_output.pci-0000_01_00.1.hdmi-stereo",
              "activePort" : {
                  "id" : "hdmi-output-0",
                  "title" : "HDMI / DisplayPort"
              },
              "device" : "GK107 HDMI Audio Controller Digital Stereo (HDMI)",
              "role" : "idle"
          }
      ],
      "audioSourceConfig" : [ 
          {
              "id" : "alsa_input.pci-0000_00_1b.0.analog-stereo",
              "activePort" : {
                  "id" : "analog-input-internal-mic",
                  "title" : "Internal Microphone"
              },
              "device" : "Built-in Audio Analog Stereo",
              "role" : "idle"
          }, 
          {
              "id" : "alsa_input.usb-046d_Logitech_Webcam_C930e_ACF992AE-02.analog-stereo",
              "serial" : "046d_Logitech_Webcam_C930e_ACF992AE",
              "activePort" : {
                  "id" : "analog-input-mic",
                  "title" : "Microphone"
              },
              "device" : "Webcam C930e Analog Stereo",
              "role" : "idle"
          }
      ],
      "displayConfig" : [ 
          {
              "id" : "DP-0",
              "port" : "DP-0",
              "x" : 0,
              "y" : 0,
              "activeMode" : {
                  "height" : 1050,
                  "width" : 1680,
                  "native" : true,
                  "rate" : 59.95
              },
              "role" : "idle",
              "parameters" : {},
              "isTouchScreen" : false,
              "isAutoGeneratedTitle" : false
          }
      ],
      "hostname" : "nrc-0060e0628ada",
      "privateIp" : "10.200.20.206",
      "rtspAuthentication" : {
          "username" : "NFyfMLGegFWGoRaxs",
          "password" : "5pYurEKqYE33QaFNG"
      },
      "status" : 3,
      "usbCameraConfig" : [ 
          {
              "id" : "00ABCDEF",
              "serial" : "00ABCDEF",
              "vendor" : "Barco",
              "model" : "001",
              "device" : "Teacher Content (Barco:001)",
              "activeFormat" : {
                  "id" : 842093913,
                  "name" : "YU12"
              },
              "activeResolution" : {
                  "width" : 960,
                  "height" : 540
              },
              "activeFramerate" : {
                  "numerator" : 1,
                  "denominator" : 30
              },
              "port" : 8700,
              "useLocally" : false,
              "record" : false,
              "stream" : false,
              "isAutoGeneratedTitle" : true,
              "agc" : false,
              "role" : "idle",
              "title" : "OVF - Idle"
          }, 
          {
              "id" : "046d_Logitech_Webcam_C930e_ACF992AE",
              "serial" : "046d_Logitech_Webcam_C930e_ACF992AE",
              "vendor" : "046d",
              "model" : "0843",
              "device" : "Logitech Webcam C930e",
              "activeFormat" : {
                  "id" : 1448695129,
                  "name" : "YUYV"
              },
              "activeResolution" : {
                  "width" : 640,
                  "height" : 480
              },
              "activeFramerate" : {
                  "numerator" : 1,
                  "denominator" : 30
              },
              "port" : 8702,
              "role" : "idle",
              "title" : "OVF - Idle",
              "isAutoGeneratedTitle" : true,
              "useLocally" : false,
              "record" : false,
              "stream" : false,
              "agc" : false
          }
      ],
      "role" : "idle",
  },
  {
      "_id" : "kbZqgxwLn4HjCfWEG",
      "meetingId" : "A7FjKihaate5LRTKv",
      "audioSinkConfig" : [ 
          {
              "id" : "alsa_output.pci-0000_00_1b.0.analog-stereo",
              "activePort" : {
                  "id" : "analog-output-speaker",
                  "title" : "Speakers"
              },
              "device" : "Built-in Audio Analog Stereo",
              "role" : "main_mix"
          }
      ],
      "audioSourceConfig" : [ 
          {
              "id" : "alsa_input.pci-0000_00_1b.0.analog-stereo",
              "activePort" : {
                  "id" : "analog-input-internal-mic",
                  "title" : "Internal Microphone"
              },
              "device" : "Built-in Audio Analog Stereo",
              "role" : "idle"
          }
      ],
      "displayConfig" : [ 
          {
              "id" : "LVDS",
              "port" : "LVDS",
              "x" : 0,
              "y" : 0,
              "activeMode" : {
                  "height" : 1080,
                  "width" : 1920,
                  "native" : true,
                  "rate" : 60.01
              },
              "role" : "main_out",
              "parameters" : {
                  "interactive" : false
              },
              "isTouchScreen" : false,
              "isAutoGeneratedTitle" : true,
              "title" : "Main Display"
          }
      ],
      "hostname" : "nrc-d067e54a95d9",
      "privateIp" : "10.200.21.40",
      "rtspAuthentication" : {
          "username" : "TGFJX8dTrYHKDhNCw",
          "password" : "o8mnEWcQ4gsEJSK3p"
      },
      "status" : 3,
      "usbCameraConfig" : [ 
          {
              "id" : "CN0CJ3P27248724CB4Y8A01_Laptop_Integrated_Webcam_FHD",
              "serial" : "CN0CJ3P27248724CB4Y8A01_Laptop_Integrated_Webcam_FHD",
              "vendor" : "1bcf",
              "model" : "280b",
              "device" : "Laptop Integrated Webcam FHD: I",
              "activeFormat" : {
                  "id" : 1448695129,
                  "name" : "YUYV"
              },
              "activeResolution" : {
                  "width" : 640,
                  "height" : 480
              },
              "activeFramerate" : {
                  "numerator" : 1,
                  "denominator" : 15
              },
              "port" : 8700,
              "useLocally" : false,
              "record" : false,
              "stream" : false,
              "isAutoGeneratedTitle" : true,
              "agc" : false,
              "role" : "idle",
              "title" : "VC - Idle"
          }, 
          {
              "id" : "00ABCDEF",
              "serial" : "00ABCDEF",
              "vendor" : "Barco",
              "model" : "001",
              "device" : "Teacher Content (Barco:001)",
              "activeFormat" : {
                  "id" : 842093913,
                  "name" : "YU12"
              },
              "activeResolution" : {
                  "width" : 960,
                  "height" : 540
              },
              "activeFramerate" : {
                  "numerator" : 1,
                  "denominator" : 30
              },
              "port" : 8702,
              "useLocally" : false,
              "record" : true,
              "stream" : true,
              "isAutoGeneratedTitle" : true,
              "agc" : false,
              "role" : "content",
              "title" : "VC - Teacher Content",
              "audioDevice" : "default"
          }
      ],
      "role" : "virtual_master",
  },
  {
      "_id" : "TiPu5hWD2vMYR2Sbz",
      "meetingId" : "A7FjKihaate5LRTKv",
      "audioSinkConfig" : [ 
          {
              "id" : "alsa_output.pci-0000_00_1b.0.analog-stereo",
              "activePort" : {
                  "id" : "analog-output-speaker",
                  "title" : "Speakers"
              },
              "device" : "Built-in Audio Analog Stereo",
              "role" : "main_mix"
          }
      ],
      "audioSourceConfig" : [ 
          {
              "id" : "alsa_input.pci-0000_00_1b.0.analog-stereo",
              "activePort" : {
                  "id" : "analog-input-internal-mic",
                  "title" : "Internal Microphone"
              },
              "device" : "Built-in Audio Analog Stereo",
              "role" : "idle"
          }, 
          {
              "id" : "alsa_input.usb-046d_Logitech_Webcam_C930e_5F5DA2AE-02.analog-stereo",
              "serial" : "046d_Logitech_Webcam_C930e_5F5DA2AE",
              "activePort" : {
                  "id" : "analog-input-mic",
                  "title" : "Microphone"
              },
              "device" : "Webcam C930e Analog Stereo",
              "role" : "idle"
          }
      ],
      "displayConfig" : [ 
          {
              "id" : "DVI-I-1",
              "port" : "DVI-I-1",
              "x" : 0,
              "y" : 0,
              "activeMode" : {
                  "height" : 1050,
                  "width" : 1680,
                  "native" : true,
                  "rate" : 59.95
              },
              "role" : "html_camera_manager",
              "parameters" : {},
              "isTouchScreen" : false,
              "isAutoGeneratedTitle" : true,
              "title" : "Camera Manager Display"
          }, 
          {
              "id" : "DVI-D-0",
              "port" : "DVI-D-0",
              "x" : 1680,
              "y" : 0,
              "activeMode" : {
                  "height" : 1050,
                  "width" : 1680,
                  "native" : true,
                  "rate" : 59.95
              },
              "role" : "idle",
              "parameters" : {},
              "isTouchScreen" : false,
              "isAutoGeneratedTitle" : true,
              "title" : "Not Used"
          }
      ],
      "hostname" : "nrc-0060e05bdf17",
      "privateIp" : "10.200.21.35",
      "rtspAuthentication" : {
          "username" : "RMAESQqiLiX8HHjM2",
          "password" : "5HAHeB9Aa7MwRmNai"
      },
      "status" : 3,
      "usbCameraConfig" : [ 
          {
              "id" : "00ABCDEF",
              "serial" : "00ABCDEF",
              "vendor" : "Barco",
              "model" : "001",
              "device" : "Teacher Content (Barco:001)",
              "activeFormat" : {
                  "id" : 842093913,
                  "name" : "YU12"
              },
              "activeResolution" : {
                  "width" : 960,
                  "height" : 540
              },
              "activeFramerate" : {
                  "numerator" : 1,
                  "denominator" : 30
              },
              "port" : 8700,
              "useLocally" : false,
              "record" : false,
              "stream" : false,
              "isAutoGeneratedTitle" : true,
              "agc" : false,
              "role" : "idle",
              "title" : "VC - Idle"
          }, 
          {
              "id" : "046d_Logitech_Webcam_C930e_5F5DA2AE",
              "serial" : "046d_Logitech_Webcam_C930e_5F5DA2AE",
              "vendor" : "046d",
              "model" : "0843",
              "device" : "Logitech Webcam C930e",
              "activeFormat" : {
                  "id" : 1448695129,
                  "name" : "YUYV"
              },
              "activeResolution" : {
                  "width" : 640,
                  "height" : 480
              },
              "activeFramerate" : {
                  "numerator" : 1,
                  "denominator" : 30
              },
              "port" : 8700,
              "role" : "presenter",
              "title" : "VC - Teacher Camera",
              "isAutoGeneratedTitle" : true,
              "useLocally" : false,
              "record" : true,
              "stream" : false,
              "agc" : false,
              "audioDevice" : "alsa_input.usb-046d_Logitech_Webcam_C930e_5F5DA2AE-02.analog-stereo"
          }
      ],
      "role" : "virtual_camera_manager",
  },
  {
      "_id" : "9kdRFwxkXA9FNB6FC",
      "meetingId" : "A7FjKihaate5LRTKv",
      "audioSinkConfig" : [ 
          {
              "id" : "alsa_output.pci-0000_00_03.0.hdmi-stereo",
              "activePort" : {
                  "id" : "hdmi-output-0",
                  "title" : "HDMI / DisplayPort"
              },
              "device" : "Built-in Audio Digital Stereo (HDMI)",
              "role" : "main_mix"
          }, 
          {
              "id" : "alsa_output.usb-Yamaha_Corporation_AG06_AG03-00.analog-stereo",
              "activePort" : {
                  "id" : "analog-output",
                  "title" : "Analog Output"
              },
              "device" : "AG06/AG03 Analog Stereo",
              "role" : "idle"
          }
      ],
      "audioSourceConfig" : [ 
          {
              "id" : "alsa_input.usb-Yamaha_Corporation_AG06_AG03-00.analog-stereo",
              "serial" : "Yamaha_Corporation_AG06_AG03",
              "activePort" : {
                  "id" : "analog-input",
                  "title" : "Analog Input"
              },
              "device" : "AG06/AG03 Analog Stereo",
              "role" : "idle"
          }
      ],
      "displayConfig" : [ 
          {
              "id" : "HDMI-1",
              "port" : "HDMI-1",
              "x" : 0,
              "y" : 0,
              "activeMode" : {
                  "height" : 1050,
                  "width" : 1680,
                  "native" : true,
                  "rate" : 59.95
              },
              "role" : "html_tile_out",
              "parameters" : {
                  "useForRole" : 1,
                  "x" : 0,
                  "y" : 0,
                  "cols" : 2,
                  "rows" : 2
              },
              "isTouchScreen" : false,
              "isAutoGeneratedTitle" : true,
              "title" : "Remote Participant Display"
          }
      ],
      "hostname" : "nrc-b8aeed7e89e9",
      "privateIp" : "10.200.21.9",
      "rtspAuthentication" : {
          "username" : "d93Tb8oT4hbfZRYCZ",
          "password" : "MQirzY9EmrNBmvYmc"
      },
      "status" : 3,
      "usbCameraConfig" : [ 
          {
              "id" : "00ABCDEF",
              "serial" : "00ABCDEF",
              "vendor" : "Barco",
              "model" : "001",
              "device" : "Teacher Content (Barco:001)",
              "activeFormat" : {
                  "id" : 842093913,
                  "name" : "YU12"
              },
              "activeResolution" : {
                  "width" : 960,
                  "height" : 540
              },
              "activeFramerate" : {
                  "numerator" : 1,
                  "denominator" : 30
              },
              "port" : 8700,
              "role" : "idle",
              "title" : "VC - Idle",
              "isAutoGeneratedTitle" : true,
              "useLocally" : false,
              "record" : false,
              "stream" : false,
              "agc" : false
          }
      ],
      "role" : "virtual_tile",
  },
  {
    "_id" : "BaPu5hWD2vMYR2Sbz",
    "meetingId" : "A7FjKihaate5LRTKv",
    "audioSinkConfig" : [ 
        {
            "id" : "alsa_output.pci-0000_00_1b.0.analog-stereo",
            "activePort" : {
                "id" : "analog-output-speaker",
                "title" : "Speakers"
            },
            "device" : "Built-in Audio Analog Stereo",
            "role" : "main_mix"
        }
    ],
    "audioSourceConfig" : [ 
        {
            "id" : "alsa_input.pci-0000_00_1b.0.analog-stereo",
            "activePort" : {
                "id" : "analog-input-internal-mic",
                "title" : "Internal Microphone"
            },
            "device" : "Built-in Audio Analog Stereo",
            "role" : "idle"
        }, 
        {
            "id" : "alsa_input.usb-046d_Logitech_Webcam_C930e_5F5DA2AE-02.analog-stereo",
            "serial" : "046d_Logitech_Webcam_C930e_5F5DA2AE",
            "activePort" : {
                "id" : "analog-input-mic",
                "title" : "Microphone"
            },
            "device" : "Webcam C930e Analog Stereo",
            "role" : "idle"
        }
    ],
    "displayConfig" : [ 
        {
            "id" : "DVI-I-1",
            "port" : "DVI-I-1",
            "x" : 0,
            "y" : 0,
            "activeMode" : {
                "height" : 1050,
                "width" : 1680,
                "native" : true,
                "rate" : 59.95
            },
            "role" : "html_camera_manager",
            "parameters" : {},
            "isTouchScreen" : false,
            "isAutoGeneratedTitle" : true,
            "title" : "Camera Manager Display"
        }, 
        {
            "id" : "DVI-D-0",
            "port" : "DVI-D-0",
            "x" : 1680,
            "y" : 0,
            "activeMode" : {
                "height" : 1050,
                "width" : 1680,
                "native" : true,
                "rate" : 59.95
            },
            "role" : "idle",
            "parameters" : {},
            "isTouchScreen" : false,
            "isAutoGeneratedTitle" : true,
            "title" : "Not Used"
        }
    ],
    "hostname" : "nrc-2060e05bdf19",
    "privateIp" : "10.200.21.39",
    "rtspAuthentication" : {
        "username" : "RMAESQqiLiX8HHjM2",
        "password" : "5HAHeB9Aa7MwRmNai"
    },
    "status" : 3,
    "usbCameraConfig" : [ 
        {
            "id" : "00ABCDEF",
            "serial" : "00ABCDEF",
            "vendor" : "Barco",
            "model" : "001",
            "device" : "Teacher Content (Barco:001)",
            "activeFormat" : {
                "id" : 842093913,
                "name" : "YU12"
            },
            "activeResolution" : {
                "width" : 960,
                "height" : 540
            },
            "activeFramerate" : {
                "numerator" : 1,
                "denominator" : 30
            },
            "port" : 8700,
            "useLocally" : false,
            "record" : false,
            "stream" : false,
            "isAutoGeneratedTitle" : true,
            "agc" : false,
            "role" : "idle",
            "title" : "VC - Idle"
        }, 
        {
            "id" : "046d_Logitech_Webcam_C930e_5F5DA2AE",
            "serial" : "046d_Logitech_Webcam_C930e_5F5DA2AE",
            "vendor" : "046d",
            "model" : "0843",
            "device" : "Logitech Webcam C930e",
            "activeFormat" : {
                "id" : 1448695129,
                "name" : "YUYV"
            },
            "activeResolution" : {
                "width" : 640,
                "height" : 480
            },
            "activeFramerate" : {
                "numerator" : 1,
                "denominator" : 30
            },
            "port" : 8700,
            "role" : "presenter",
            "title" : "VC - Teacher Camera",
            "isAutoGeneratedTitle" : true,
            "useLocally" : false,
            "record" : false,
            "stream" : false,
            "agc" : false,
        }
    ],
    "role" : "virtual_camera_manager",
  },
  {
      "_id" : "0dkRFwxkXA9FNB6CF",
      "meetingId" : "A7FjKihaate5LRTKv",
      "audioSinkConfig" : [ 
          {
              "id" : "alsa_output.pci-0000_00_03.0.hdmi-stereo",
              "activePort" : {
                  "id" : "hdmi-output-0",
                  "title" : "HDMI / DisplayPort"
              },
              "device" : "Built-in Audio Digital Stereo (HDMI)",
              "role" : "main_mix"
          }, 
          {
              "id" : "alsa_output.usb-Yamaha_Corporation_AG06_AG03-00.analog-stereo",
              "activePort" : {
                  "id" : "analog-output",
                  "title" : "Analog Output"
              },
              "device" : "AG06/AG03 Analog Stereo",
              "role" : "idle"
          }
      ],
      "audioSourceConfig" : [ 
          {
              "id" : "alsa_input.usb-Yamaha_Corporation_AG06_AG03-00.analog-stereo",
              "serial" : "Yamaha_Corporation_AG06_AG03",
              "activePort" : {
                  "id" : "analog-input",
                  "title" : "Analog Input"
              },
              "device" : "AG06/AG03 Analog Stereo",
              "role" : "idle"
          }
      ],
      "displayConfig" : [ 
          {
              "id" : "HDMI-1",
              "port" : "HDMI-1",
              "x" : 0,
              "y" : 0,
              "activeMode" : {
                  "height" : 1050,
                  "width" : 1680,
                  "native" : true,
                  "rate" : 59.95
              },
              "role" : "html_tile_out",
              "parameters" : {
                  "useForRole" : 1,
                  "x" : 0,
                  "y" : 0,
                  "cols" : 2,
                  "rows" : 2
              },
              "isTouchScreen" : false,
              "isAutoGeneratedTitle" : true,
              "title" : "Remote Participant Display"
          }
      ],
      "hostname" : "nrc-poieed7e89e9",
      "privateIp" : "10.200.20.15",
      "rtspAuthentication" : {
          "username" : "blaib8oT4hbfZRYCZ",
          "password" : "TrutiY9EmrNBmvYmc"
      },
      "status" : 3,
      "usbCameraConfig" : [ 
          {
              "id" : "00ABCDEF",
              "serial" : "00ABCDEF",
              "vendor" : "Barco",
              "model" : "001",
              "device" : "Teacher Content (Barco:001)",
              "activeFormat" : {
                  "id" : 842093913,
                  "name" : "YU12"
              },
              "activeResolution" : {
                  "width" : 960,
                  "height" : 540
              },
              "activeFramerate" : {
                  "numerator" : 1,
                  "denominator" : 30
              },
              "port" : 8700,
              "role" : "idle",
              "title" : "VC - Idle",
              "isAutoGeneratedTitle" : true,
              "useLocally" : false,
              "record" : false,
              "stream" : false,
              "agc" : false
          }
      ],
      "role" : "virtual_tile",
    },
  ];


  const devicesOverflowLecture = [
    {
        "_id" : "M3NmAcvCALHY7sZv3",
        "audioSinkConfig" : [ 
            {
                "id" : "alsa_output.pci-0000_00_1b.0.analog-stereo",
                "activePort" : {
                    "id" : "analog-output-speaker",
                    "title" : "Speakers"
                },
                "device" : "Built-in Audio Analog Stereo",
                "role" : "main_mix"
            }, 
            {
                "id" : "alsa_output.pci-0000_01_00.1.hdmi-stereo",
                "activePort" : {
                    "id" : "hdmi-output-0",
                    "title" : "HDMI / DisplayPort"
                },
                "device" : "GK107 HDMI Audio Controller Digital Stereo (HDMI)",
                "role" : "idle"
            }
        ],
        "audioSourceConfig" : [ 
            {
                "id" : "alsa_input.pci-0000_00_1b.0.analog-stereo",
                "activePort" : {
                    "id" : "analog-input-internal-mic",
                    "title" : "Internal Microphone"
                },
                "device" : "Built-in Audio Analog Stereo",
                "role" : "idle"
            }, 
            {
                "id" : "alsa_input.usb-046d_Logitech_Webcam_C930e_ACF992AE-02.analog-stereo",
                "serial" : "046d_Logitech_Webcam_C930e_ACF992AE",
                "activePort" : {
                    "id" : "analog-input-mic",
                    "title" : "Microphone"
                },
                "device" : "Webcam C930e Analog Stereo",
                "role" : "idle"
            }
        ],
        "displayConfig" : [ 
            {
                "id" : "DP-0",
                "port" : "DP-0",
                "x" : 0,
                "y" : 0,
                "activeMode" : {
                    "height" : 1050,
                    "width" : 1680,
                    "native" : true,
                    "rate" : 59.95
                },
                "role" : "slave_combined_from_master",
                "parameters" : {},
                "isTouchScreen" : false,
                "isAutoGeneratedTitle" : true,
                "title" : "Combined Monitor"
            }
        ],
        "hostname" : "nrc-0060e0628ada",
        "meetingId" : "xG9Y6uhLSkSPxhZNc",
        "privateIp" : "10.200.20.206",
        "rtspAuthentication" : {
            "username" : "MqmoQ4ye4BC829BBn",
            "password" : "5EA82SeFxov7SSdTN"
        },
        "status" : 3,
        "usbCameraConfig" : [ 
            {
                "id" : "00ABCDEF",
                "serial" : "00ABCDEF",
                "vendor" : "Barco",
                "model" : "001",
                "device" : "Teacher Content (Barco:001)",
                "activeFormat" : {
                    "id" : 842093913,
                    "name" : "YU12"
                },
                "activeResolution" : {
                    "width" : 960,
                    "height" : 540
                },
                "activeFramerate" : {
                    "numerator" : 1,
                    "denominator" : 30
                },
                "port" : 8700,
                "useLocally" : false,
                "record" : false,
                "stream" : false,
                "isAutoGeneratedTitle" : true,
                "agc" : false,
                "role" : "idle",
                "title" : "OVF - Idle"
            }, 
            {
                "id" : "046d_Logitech_Webcam_C930e_ACF992AE",
                "serial" : "046d_Logitech_Webcam_C930e_ACF992AE",
                "vendor" : "046d",
                "model" : "0843",
                "device" : "Logitech Webcam C930e",
                "activeFormat" : {
                    "id" : 1448695129,
                    "name" : "YUYV"
                },
                "activeResolution" : {
                    "width" : 640,
                    "height" : 480
                },
                "activeFramerate" : {
                    "numerator" : 1,
                    "denominator" : 30
                },
                "port" : 8702,
                "role" : "room",
                "title" : "OVF - Room Camera",
                "isAutoGeneratedTitle" : true,
                "useLocally" : false,
                "record" : false,
                "stream" : true,
                "agc" : false,
                "audioDevice" : "alsa_input.usb-046d_Logitech_Webcam_C930e_ACF992AE-02.analog-stereo"
            }
        ],
        "role" : "overflow_slave",
    },
    {
        "_id" : "kbZqgxwLn4HjCfWEG",
        "audioSinkConfig" : [ 
            {
                "id" : "alsa_output.pci-0000_00_1b.0.analog-stereo",
                "activePort" : {
                    "id" : "analog-output-speaker",
                    "title" : "Speakers"
                },
                "device" : "Built-in Audio Analog Stereo",
                "role" : "idle"
            }
        ],
        "audioSourceConfig" : [ 
            {
                "id" : "alsa_input.pci-0000_00_1b.0.analog-stereo",
                "activePort" : {
                    "id" : "analog-input-internal-mic",
                    "title" : "Internal Microphone"
                },
                "device" : "Built-in Audio Analog Stereo",
                "role" : "idle"
            }
        ],
        "displayConfig" : [ 
            {
                "id" : "LVDS",
                "port" : "LVDS",
                "x" : 0,
                "y" : 0,
                "activeMode" : {
                    "height" : 1080,
                    "width" : 1920,
                    "native" : true,
                    "rate" : 60.01
                },
                "role" : "main_out",
                "parameters" : {
                    "interactive" : false
                },
                "isTouchScreen" : false,
                "isAutoGeneratedTitle" : true,
                "title" : "Main Display"
            }
        ],
        "hostname" : "nrc-d067e54a95d9",
        "meetingId" : "xG9Y6uhLSkSPxhZNc",
        "privateIp" : "10.200.21.40",
        "rtspAuthentication" : {
            "username" : "t5yQ9jmhYGmYY923M",
            "password" : "gLYdw9G5WumZbY4fY"
        },
        "status" : 3,
        "usbCameraConfig" : [ 
            {
                "id" : "CN0CJ3P27248724CB4Y8A01_Laptop_Integrated_Webcam_FHD",
                "serial" : "CN0CJ3P27248724CB4Y8A01_Laptop_Integrated_Webcam_FHD",
                "vendor" : "1bcf",
                "model" : "280b",
                "device" : "Laptop Integrated Webcam FHD: I",
                "activeFormat" : {
                    "id" : 1448695129,
                    "name" : "YUYV"
                },
                "activeResolution" : {
                    "width" : 640,
                    "height" : 480
                },
                "activeFramerate" : {
                    "numerator" : 1,
                    "denominator" : 15
                },
                "port" : 8700,
                "role" : "idle",
                "title" : "VC - Idle",
                "isAutoGeneratedTitle" : true,
                "useLocally" : false,
                "record" : false,
                "stream" : false,
                "agc" : false
            }, 
            {
                "id" : "00ABCDEF",
                "serial" : "00ABCDEF",
                "vendor" : "Barco",
                "model" : "001",
                "device" : "Teacher Content (Barco:001)",
                "activeFormat" : {
                    "id" : 842093913,
                    "name" : "YU12"
                },
                "activeResolution" : {
                    "width" : 960,
                    "height" : 540
                },
                "activeFramerate" : {
                    "numerator" : 1,
                    "denominator" : 30
                },
                "port" : 8702,
                "role" : "idle",
                "title" : "VC - Idle",
                "isAutoGeneratedTitle" : true,
                "useLocally" : false,
                "record" : false,
                "stream" : false,
                "agc" : false
            }
        ],
        "role" : "overflow_master",
    },
    {
        "_id" : "TiPu5hWD2vMYR2Sbz",
        "audioSinkConfig" : [ 
            {
                "id" : "alsa_output.pci-0000_00_1b.0.analog-stereo",
                "activePort" : {
                    "id" : "analog-output-speaker",
                    "title" : "Speakers"
                },
                "device" : "Built-in Audio Analog Stereo",
                "role" : "main_mix"
            }
        ],
        "audioSourceConfig" : [ 
            {
                "id" : "alsa_input.pci-0000_00_1b.0.analog-stereo",
                "activePort" : {
                    "id" : "analog-input-internal-mic",
                    "title" : "Internal Microphone"
                },
                "device" : "Built-in Audio Analog Stereo",
                "role" : "idle"
            }, 
            {
                "id" : "alsa_input.usb-046d_Logitech_Webcam_C930e_5F5DA2AE-02.analog-stereo",
                "serial" : "046d_Logitech_Webcam_C930e_5F5DA2AE",
                "activePort" : {
                    "id" : "analog-input-mic",
                    "title" : "Microphone"
                },
                "device" : "Webcam C930e Analog Stereo",
                "role" : "idle"
            }
        ],
        "displayConfig" : [ 
            {
                "id" : "DVI-I-1",
                "port" : "DVI-I-1",
                "x" : 0,
                "y" : 0,
                "activeMode" : {
                    "height" : 1050,
                    "width" : 1680,
                    "native" : true,
                    "rate" : 59.95
                },
                "role" : "rtsp_camera_manager",
                "parameters" : {},
                "isTouchScreen" : false,
                "isAutoGeneratedTitle" : true,
                "title" : "Camera Manager Display"
            }, 
            {
                "id" : "DVI-D-0",
                "port" : "DVI-D-0",
                "x" : 1680,
                "y" : 0,
                "activeMode" : {
                    "height" : 1050,
                    "width" : 1680,
                    "native" : true,
                    "rate" : 59.95
                },
                "role" : "idle",
                "parameters" : {},
                "isTouchScreen" : false,
                "isAutoGeneratedTitle" : true,
                "title" : "Not Used"
            }
        ],
        "hostname" : "nrc-0060e05bdf17",
        "meetingId" : "xG9Y6uhLSkSPxhZNc",
        "privateIp" : "10.200.21.35",
        "rtspAuthentication" : {
            "username" : "WJdfMkjmuDjZYL7tR",
            "password" : "4rrkbtmY4RQ2AoHf6"
        },
        "status" : 3,
        "usbCameraConfig" : [ 
            {
                "id" : "046d_Logitech_Webcam_C930e_5F5DA2AE",
                "serial" : "046d_Logitech_Webcam_C930e_5F5DA2AE",
                "vendor" : "046d",
                "model" : "0843",
                "device" : "Logitech Webcam C930e",
                "activeFormat" : {
                    "id" : 1448695129,
                    "name" : "YUYV"
                },
                "activeResolution" : {
                    "width" : 640,
                    "height" : 480
                },
                "activeFramerate" : {
                    "numerator" : 1,
                    "denominator" : 30
                },
                "port" : 8700,
                "useLocally" : false,
                "record" : true,
                "stream" : true,
                "isAutoGeneratedTitle" : true,
                "agc" : true,
                "role" : "presenter",
                "title" : "VC - Teacher Camera",
                "audioDevice" : "alsa_input.usb-046d_Logitech_Webcam_C930e_5F5DA2AE-02.analog-stereo"
            }, 
            {
                "id" : "00ABCDEF",
                "serial" : "00ABCDEF",
                "vendor" : "Barco",
                "model" : "001",
                "device" : "Teacher Content (Barco:001)",
                "activeFormat" : {
                    "id" : 842093913,
                    "name" : "YU12"
                },
                "activeResolution" : {
                    "width" : 960,
                    "height" : 540
                },
                "activeFramerate" : {
                    "numerator" : 1,
                    "denominator" : 30
                },
                "port" : 8700,
                "useLocally" : false,
                "record" : false,
                "stream" : false,
                "isAutoGeneratedTitle" : true,
                "agc" : false,
                "role" : "idle",
                "title" : "VC - Idle"
            }
        ],
        "role" : "camera_manager_master",
    },
    {
        "_id" : "9kdRFwxkXA9FNB6FC",
        "audioSinkConfig" : [ 
            {
                "id" : "alsa_output.pci-0000_00_03.0.hdmi-stereo",
                "activePort" : {
                    "id" : "hdmi-output-0",
                    "title" : "HDMI / DisplayPort"
                },
                "device" : "Built-in Audio Digital Stereo (HDMI)",
                "role" : "idle"
            }, 
            {
                "id" : "alsa_output.usb-Yamaha_Corporation_AG06_AG03-00.analog-stereo",
                "activePort" : {
                    "id" : "analog-output",
                    "title" : "Analog Output"
                },
                "device" : "AG06/AG03 Analog Stereo",
                "role" : "idle"
            }
        ],
        "audioSourceConfig" : [ 
            {
                "id" : "alsa_input.usb-Yamaha_Corporation_AG06_AG03-00.analog-stereo",
                "serial" : "Yamaha_Corporation_AG06_AG03",
                "activePort" : {
                    "id" : "analog-input",
                    "title" : "Analog Input"
                },
                "device" : "AG06/AG03 Analog Stereo",
                "role" : "idle"
            }
        ],
        "displayConfig" : [ 
            {
                "id" : "HDMI-1",
                "port" : "HDMI-1",
                "x" : 0,
                "y" : 0,
                "activeMode" : {
                    "height" : 1050,
                    "width" : 1680,
                    "native" : true,
                    "rate" : 59.95
                },
                "role" : "overflow_remote",
                "parameters" : {},
                "isTouchScreen" : false,
                "isAutoGeneratedTitle" : true,
                "title" : "Combined Remote Auditorium Monitor"
            }
        ],
        "hostname" : "nrc-b8aeed7e89e9",
        "meetingId" : "xG9Y6uhLSkSPxhZNc",
        "privateIp" : "10.200.21.9",
        "rtspAuthentication" : {
            "username" : "NwCZ8yvZAoP7S4KtY",
            "password" : "LW2P8fhKzwJLtuAtw"
        },
        "status" : 3,
        "usbCameraConfig" : [ 
            {
                "id" : "00ABCDEF",
                "serial" : "00ABCDEF",
                "vendor" : "Barco",
                "model" : "001",
                "device" : "Teacher Content (Barco:001)",
                "activeFormat" : {
                    "id" : 842093913,
                    "name" : "YU12"
                },
                "activeResolution" : {
                    "width" : 960,
                    "height" : 540
                },
                "activeFramerate" : {
                    "numerator" : 1,
                    "denominator" : 30
                },
                "port" : 8700,
                "role" : "idle",
                "title" : "VC - Idle",
                "isAutoGeneratedTitle" : true,
                "useLocally" : false,
                "record" : false,
                "stream" : false,
                "agc" : false
            }
        ],
        "role" : "confidence_overflow_master",
    },
  ];

  
  const expectedResultHybridLecture = [
    [], // idle device should not pull any stream
    [], // virtual master should not pull any stream
    [ // Camera mgr pull from master and the 2 tiles
      {
        _id: 'kbZqgxwLn4HjCfWEG',
        meetingId : "A7FjKihaate5LRTKv",
        role: 'virtual_master',
        privateIp: '10.200.21.40',
        hostname: 'nrc-d067e54a95d9',
        username: 'TGFJX8dTrYHKDhNCw',
        password: 'o8mnEWcQ4gsEJSK3p',
        sinkname: 'CA_21_40'
      },
      {
        _id: '9kdRFwxkXA9FNB6FC',
        meetingId : "A7FjKihaate5LRTKv",
        role: 'virtual_tile',
        privateIp: '10.200.21.9',
        hostname: 'nrc-b8aeed7e89e9',
        username: 'd93Tb8oT4hbfZRYCZ',
        password: 'MQirzY9EmrNBmvYmc',
        sinkname: 'TA_21_9'
      },
      {
        _id: '0dkRFwxkXA9FNB6CF',
        meetingId : "A7FjKihaate5LRTKv",
        role: 'virtual_tile',
        privateIp: '10.200.20.15',
        hostname: 'nrc-poieed7e89e9',
        username: 'blaib8oT4hbfZRYCZ',
        password: 'TrutiY9EmrNBmvYmc',
        sinkname: 'TA_20_15'
      }
    ],
    [ // Tile 1 pulls from tile 2 (= intertile)
      {
        _id: '0dkRFwxkXA9FNB6CF',
        meetingId : "A7FjKihaate5LRTKv",
        role: 'virtual_tile',
        privateIp: '10.200.20.15',
        hostname: 'nrc-poieed7e89e9',
        username: 'blaib8oT4hbfZRYCZ',
        password: 'TrutiY9EmrNBmvYmc',
        sinkname: 'TA_20_15'
      }
    ],
    [], // camera mgr without camera that has audio does should not pull any stream
    [ // Tile 2 pulls from tile 1 (= intertile)
      {
        _id: '9kdRFwxkXA9FNB6FC',
        meetingId : "A7FjKihaate5LRTKv",
        role: 'virtual_tile',
        privateIp: '10.200.21.9',
        hostname: 'nrc-b8aeed7e89e9',
        username: 'd93Tb8oT4hbfZRYCZ',
        password: 'MQirzY9EmrNBmvYmc',
        sinkname: 'TA_21_9'
      }
    ]
  ];

  const expectedResultHybridLectureWithAuxInInput = [
    [], // idle device should not pull any stream
    [], // virtual master should not pull any stream
    [], // Camera mgr should not pull as it has a loopthrough input
    [], // Tile 1 should not pull any stream
    [], // camera mgr without camera that has audio does should not pull any stream
    [], // Tile 2 should not pull any stream
  ];

  const expectedResultOverflowLecture = [
    [], // Overflow master should not pull any stream
    [], // Overflow slave should not pull any stream
    [ // Camera mgr pull from master and the confidence monitor
      {
        _id: 'kbZqgxwLn4HjCfWEG',
        role: 'overflow_master',
        privateIp: '10.200.21.40',
        hostname: 'nrc-d067e54a95d9',
        meetingId: 'xG9Y6uhLSkSPxhZNc',
        username: 't5yQ9jmhYGmYY923M',
        password: 'gLYdw9G5WumZbY4fY',
        sinkname: 'CA_21_40'
      },
      {
        _id: '9kdRFwxkXA9FNB6FC',
        role: 'confidence_overflow_master',
        privateIp: '10.200.21.9',
        hostname: 'nrc-b8aeed7e89e9',
        meetingId: 'xG9Y6uhLSkSPxhZNc',
        username: 'NwCZ8yvZAoP7S4KtY',
        password: 'LW2P8fhKzwJLtuAtw',
        sinkname: 'CMA_21_9'
      }
    ],
    [] // Confidence monitor should not pull any stream
  ];

  const expectedResultOverflowLectureWithLoopthroughInput = [
    [], // Overflow master should not pull any stream
    [], // Overflow slave should not pull any stream
    [], // Camera mgr should not pull as it has a loopthrough input
    [] // Confidence monitor should not pull any stream
  ];
  

  beforeEach(() => {
  });

  after(() => {});

  it('Should generate the right list of devices to pull audio from (Hybrid type lecture)', () => {
    let resultHybridLecture = [];
    devicesHybridLecture.forEach(d => resultHybridLecture.push(generateListOfDevicesToPullAudioFrom(devicesHybridLecture, d, false, dummyLogger)));
    assert.deepEqual(resultHybridLecture, expectedResultHybridLecture);
  });

  it('Should generate the right list of devices to pull audio from (Overflow type lecture)', () => {
    let resultOverflowLecture = [];
    devicesOverflowLecture.forEach(d => resultOverflowLecture.push(generateListOfDevicesToPullAudioFrom(devicesOverflowLecture, d, false, dummyLogger)));
    assert.deepEqual(resultOverflowLecture, expectedResultOverflowLecture);
  });
});