import V4LStream from './v4lStream';

/**
 * v4l stream
 *
 * Creates a stream that takes its input from the x screen buffer
 * and sends its output to a v4l loopback device.
 * As such a "virtual USB camera (framegrabber)" is created.
 *
 */
class V4LScreenStream extends V4LStream {
  constructor(options) {
    super(options);
    this.origin = options.origin;
    this._log('info', `Creating v4l screenscrape stream for [${this.origin.x},${this.origin.y},${this.origin.x + this.resolution.width},${this.origin.y + this.resolution.height}]`);
  }

  get streamType() {
    return 'V4LScreen';
  }

  get args() {
    let args = `ximagesrc display-name=:0 use-damage=0 startx=${this.origin.x} starty=${this.origin.y} endx=${this.origin.x + this.resolution.width - 1} endy=${this.origin.y + this.resolution.height - 1}`;
    args += ` ! video/x-raw,format=BGRx,framerate=${this.framerate.denominator}/${this.framerate.numerator}`;
    args += ` ! vaapipostproc width=${this.v4lResolution.width} height=${this.v4lResolution.height} format=i420`;
    args += ' ! videoconvert';
    args += ' ! video/x-raw, colorimetry=2:4:7:1';
    args += ' ! queue';
    args += ' ! identity drop-allocation=1';
    args += ` ! v4l2sink device=${this.v4lPath}`;
    return args;
  }

  equals(stream) {
    return super.equals(stream)
      && stream instanceof V4LScreenStream
      && (this.origin.x === stream.origin.x)
      && (this.origin.y === stream.origin.y);
  }
}

export default V4LScreenStream;
