import V4LStream from './v4lStream';

/**
 * v4l stream
 *
 * Creates a stream that takes its input from shared memory
 * (Cinergy) and sends its output to a v4l loopback device.
 * As such a "virtual USB camera (framegrabber)" is created.
 *
 */
class V4LShmStream extends V4LStream {
  constructor(options) {
    super(options);
    this.path = options.path;
    this._log('info', `Creating v4l shm stream for path ${this.path} at ${this.v4lPath}`);
  }

  get streamType() {
    return 'V4LShm';
  }

  get args() {
    let args = `shmsrc socket-path=${this.path} is-live=true do-timestamp=true`;
    args += ` ! "video/x-raw, framerate=${this.framerate.denominator}/${this.framerate.numerator}, width=${this.resolution.width}, height=${this.resolution.height}, format=I420, pixel-aspect-ratio=1/1, interlace-mode=progressive, colorimetry=2:4:7:1"`;
    args += ' ! videoscale';
    args += ` ! "video/x-raw, width=${this.v4lResolution.width}, height=${this.v4lResolution.height}"`;
    args += ' ! queue';
    args += ' ! identity drop-allocation=1';
    args += ` ! v4l2sink device=${this.v4lPath}`;
    return args;
  }

  equals(stream) {
    return super.equals(stream)
      && stream instanceof V4LShmStream
      && (this.path === stream.path);
  }
}

export default V4LShmStream;
