import Stream from './stream';

/**
 * v4l stream
 *
 * Creates a stream and sends its output to a v4l loopback device.
 * As such a "virtual USB camera (framegrabber)" is created.
 *
 */
class V4LStream extends Stream {
  constructor({
    resolution, framerate, v4lPath, v4lResolution,
  }) {
    super({ id: `${v4lPath}-v4l` });
    this.resolution = resolution;
    this.framerate = framerate;
    this.v4lPath = v4lPath;
    this.v4lResolution = v4lResolution;
  }

  get command() {
    return 'gst-launch-1.0';
  }

  get streamType() {
    return 'V4L';
  }

  equals(stream) {
    return stream instanceof V4LStream
      && (this.resolution.width === stream.resolution.width)
      && (this.resolution.height === stream.resolution.height)
      && (this.framerate.numerator === stream.framerate.numerator)
      && (this.framerate.denominator === stream.framerate.denominator)
      && (this.v4lPath === stream.v4lPath)
      && (this.v4lResolution.numerator === stream.v4lResolution.numerator)
      && (this.v4lResolution.denominator === stream.v4lResolution.denominator);
  }
}

export default V4LStream;
