import * as fs from 'fs-extra';

import Bridge from '../bridge';
import { commands } from '../../../services/ddp/constants';
import { CONFIG_FILE } from '../../../files';

/**
 * The Unpair bridge handles the identification messages
 * It glues blue with the messenger service
 */
class UnpairBridge extends Bridge {
  get supportedRPCCommands() {
    return [commands.UNPAIR];
  }

  /**
   * Act on incoming client RPCs
   * RPC's are useful for one-time actions that are not part of the application state.
   *
   * @param rpc
   * @returns {*}
   * @private
   */
  _onRpcAdded(rpc) {
    this._log('verbose', `Handling rpc ${JSON.stringify(rpc, null, ' ')}`, { serviceLog: true });
    switch (rpc.command) {
    case commands.UNPAIR:
      setTimeout(() => this._unpair(), 5000);
      break;
    default:
    }
  }

  _unpair() {
    this._log('warn', 'About to unpair device');
    try {
      fs.unlinkSync(CONFIG_FILE);
    } catch (error) {
      this._log('error', 'Could not unpair device', error);
      return error;
    }
    // Let monit restart edu_proxy in order to
    // show the "Ready for pairing" message.
    process.exit(-1);
  }
}

export default UnpairBridge;
