/*
eslint-disable
*/
import { assert } from 'chai';
import { calculateAudioConfig } from './calculatePAConfig';


describe('Test generation of configuration for pulse audio', () => {
  let dummyLogger;
  const device = 
  {
    "_id": "p2R636AnTNvPjGZAx",
    "audio": {
      "mute": false,
      "volume": 87
    },
    "audioSinkConfig": [
      {
        "id": "alsa_output.pci-0000_00_1b.0.analog-stereo",
        "activePort": {
          "id": "analog-output-lineout",
          "title": "Line Out"
        },
        "device": "Built-in Audio Analog Stereo",
        "role": "main_mix"
      }
    ],
    "audioSinks": [
      {
        "id": "alsa_output.pci-0000_00_1b.0.analog-stereo",
        "activePort": {
          "id": "analog-output-lineout",
          "title": "Line Out"
        },
        "device": "Built-in Audio Analog Stereo",
        "ports": [
          {
            "id": "analog-output-lineout",
            "title": "Line Out"
          },
          {
            "id": "analog-output-speaker",
            "title": "Speakers"
          }
        ],
        "defaultVolume": 100
      }
    ],
    "audioSinksLatest": [
      {
        "id": "alsa_output.pci-0000_00_1b.0.analog-stereo",
        "activePort": {
          "id": "analog-output-lineout",
          "title": "Line Out"
        },
        "device": "Built-in Audio Analog Stereo",
        "ports": [
          {
            "id": "analog-output-lineout",
            "title": "Line Out"
          },
          {
            "id": "analog-output-speaker",
            "title": "Speakers"
          }
        ],
        "defaultVolume": 100
      }
    ],
    "audioSourceConfig": [
      {
        "id": "alsa_input.pci-0000_00_1b.0.analog-stereo",
        "activePort": {
          "id": "analog-input-internal-mic",
          "title": "Internal Microphone"
        },
        "device": "Built-in Audio Analog Stereo",
        "role": "aec_ref_in"
      },
      {
        "id": "alsa_input.usb-046d_Logitech_Webcam_C930e_5F5DA2AE-02.analog-stereo",
        "serial": "046d_Logitech_Webcam_C930e_5F5DA2AE",
        "activePort": {
          "id": "analog-input-mic",
          "title": "Microphone"
        },
        "device": "Webcam C930e Analog Stereo",
        "role": "idle"
      }
    ],
    "audioSources": [
      {
        "id": "alsa_input.usb-046d_Logitech_Webcam_C930e_5F5DA2AE-02.analog-stereo",
        "serial": "046d_Logitech_Webcam_C930e_5F5DA2AE",
        "activePort": {
          "id": "analog-input-mic",
          "title": "Microphone"
        },
        "device": "Webcam C930e Analog Stereo",
        "ports": [
          {
            "id": "analog-input-mic",
            "title": "Microphone"
          }
        ],
        "defaultVolume": 90
      },
      {
        "id": "alsa_input.pci-0000_00_1b.0.analog-stereo",
        "activePort": {
          "id": "analog-input-internal-mic",
          "title": "Internal Microphone"
        },
        "device": "Built-in Audio Analog Stereo",
        "ports": [
          {
            "id": "analog-input-internal-mic",
            "title": "Internal Microphone"
          },
          {
            "id": "analog-input-mic",
            "title": "Microphone"
          },
          {
            "id": "analog-input-linein",
            "title": "Line In"
          }
        ],
        "defaultVolume": 10
      }
    ],
    "audioSourcesLatest": [
      {
        "id": "alsa_input.usb-046d_Logitech_Webcam_C930e_5F5DA2AE-02.analog-stereo",
        "serial": "046d_Logitech_Webcam_C930e_5F5DA2AE",
        "activePort": {
          "id": "analog-input-mic",
          "title": "Microphone"
        },
        "device": "Webcam C930e Analog Stereo",
        "ports": [
          {
            "id": "analog-input-mic",
            "title": "Microphone"
          }
        ],
        "defaultVolume": 90
      },
      {
        "id": "alsa_input.pci-0000_00_1b.0.analog-stereo",
        "activePort": {
          "id": "analog-input-internal-mic",
          "title": "Internal Microphone"
        },
        "device": "Built-in Audio Analog Stereo",
        "ports": [
          {
            "id": "analog-input-internal-mic",
            "title": "Internal Microphone"
          },
          {
            "id": "analog-input-mic",
            "title": "Microphone"
          },
          {
            "id": "analog-input-linein",
            "title": "Line In"
          }
        ],
        "defaultVolume": 10
      }
    ],
    "configured": true,
    "contentDefaultVolume": 100,
    "meetingId": "WHxq8NnfoiKfLTsqN",
    "role": "collaboration_overflow_master",
    "streamRemoteAudio": false,
    "teleportStatus": 1,
    "time": {
      "ntpServers": [
        "nntp.barco.com"
      ],
      "zone": "Europe/Brussels",
      "_metadata": {
        "master": "peFWGuqtABoeo2z9d",
        "isMaster": false
      }
    },
    "title": "NRC",
    "type": "display_node",
    "usbCameraConfig": [
      {
        "id": "046d_Logitech_Webcam_C930e_5F5DA2AE",
        "serial": "046d_Logitech_Webcam_C930e_5F5DA2AE",
        "vendor": "046d",
        "model": "0843",
        "device": "Logitech Webcam C930e",
        "activeFormat": {
          "id": 1448695129,
          "name": "YUYV"
        },
        "activeResolution": {
          "width": 1280,
          "height": 720
        },
        "activeFramerate": {
          "numerator": 1,
          "denominator": 10
        },
        "port": 8702,
        "isAutoGeneratedTitle": true,
        "stream": true,
        "agc": true,
        "useLocally": false,
        "role": "presenter",
        "title": "VC - Teacher Camera",
        "audioDevice": "alsa_input.usb-046d_Logitech_Webcam_C930e_5F5DA2AE-02.analog-stereo"
      },
      {
        "id": "00ABCDEF",
        "serial": "00ABCDEF",
        "vendor": "Barco",
        "model": "001",
        "device": "Teacher Content (Barco:001)",
        "activeFormat": {
          "id": 842093913,
          "name": "YU12"
        },
        "activeResolution": {
          "width": 960,
          "height": 540
        },
        "activeFramerate": {
          "numerator": 1,
          "denominator": 30
        },
        "port": 8700,
        "isAutoGeneratedTitle": true,
        "stream": false,
        "agc": false,
        "useLocally": false,
        "role": "idle",
        "title": "VC - Idle"
      }
    ],
    "version": "1.8.0-b57",
    "status": 2
  }
  
  const expectedAudioConfig = 
  {
    "routing": [
      {
        "module": "module-combine-sink",
        "arguments": {
          "sink_name": "main_mix",
          "slaves": [
            "alsa_output.pci-0000_00_1b.0.analog-stereo"
          ]
        }
      },
      {
        "module": "module-combine-sink",
        "arguments": {
          "sink_name": "local_mix",
          "slaves": [
            "main_mix"
          ]
        }
      },
      {
        "module": "module-combine-sink",
        "arguments": {
          "sink_name": "remote_mix",
          "slaves": [
            "main_mix"
          ]
        }
      },
      {
        "module": "module-null-sink",
        "arguments": {
          "sink_name": "mic_mix_0"
        }
      },
      {
        "module": "module-ladspa-sink",
        "arguments": {
          "sink_name": "agc_snk_0",
          "sink_master": "mic_mix_0",
          "plugin": "sc4m_1916",
          "label": "sc4m",
          "control": "0,10,150,-18,5,5,3"
        }
      },
      {
        "module": "module-loopback",
        "arguments": {
          "source": "alsa_input.usb-046d_Logitech_Webcam_C930e_5F5DA2AE-02.analog-stereo",
          "sink": "agc_snk_0"
        }
      },
      {
        "module": "module-loopback",
        "arguments": {
          "source": "alsa_input.pci-0000_00_1b.0.analog-stereo",
          "sink": "main_mix",
          "latency_msec": "20"
        }
      }
    ],
    "currentDeviceRole": "collaboration_overflow_master",
    "streamRemoteAudio": false,
    "localMixNeeded": true,
    "remoteMixNeeded": true,
    "mainMixNeeded": true,
    "recordingMixNeeded": false,
    "meetingId": "WHxq8NnfoiKfLTsqN",
    "calibrationState": "off",
    "defaultSource": "main_mix.monitor",
    "defaultSink": "local_mix",
    "interTileMixSink": "local_mix",
    "contentSharingSink": "local_mix",
    "remoteSourceSink": "remote_mix",
    "sinkForReferenceAudioSample": {
      "teacher": "nullsink",
      "content": "main_mix",
      "background": "nullsink"
    },
    "rtspContentStreamSource": "local_mix.monitor",
    "sourcesWithRole": {
      "idle": [
        {
          "id": "alsa_input.usb-046d_Logitech_Webcam_C930e_5F5DA2AE-02.analog-stereo",
          "serial": "046d_Logitech_Webcam_C930e_5F5DA2AE",
          "activePort": {
            "id": "analog-input-mic",
            "title": "Microphone"
          },
          "device": "Webcam C930e Analog Stereo",
          "ports": [
            {
              "id": "analog-input-mic",
              "title": "Microphone"
            }
          ],
          "defaultVolume": 90
        }
      ],
      "aux_in": [],
      "aec_mic_in": [],
      "aec_ref_in": [
        {
          "id": "alsa_input.pci-0000_00_1b.0.analog-stereo",
          "activePort": {
            "id": "analog-input-internal-mic",
            "title": "Internal Microphone"
          },
          "device": "Built-in Audio Analog Stereo",
          "ports": [
            {
              "id": "analog-input-internal-mic",
              "title": "Internal Microphone"
            },
            {
              "id": "analog-input-mic",
              "title": "Microphone"
            },
            {
              "id": "analog-input-linein",
              "title": "Line In"
            }
          ],
          "defaultVolume": 10
        }
      ]
    },
    "sinksWithRole": {
      "idle": [],
      "main_mix": [
        {
          "id": "alsa_output.pci-0000_00_1b.0.analog-stereo",
          "activePort": {
            "id": "analog-output-lineout",
            "title": "Line Out"
          },
          "device": "Built-in Audio Analog Stereo",
          "ports": [
            {
              "id": "analog-output-lineout",
              "title": "Line Out"
            },
            {
              "id": "analog-output-speaker",
              "title": "Speakers"
            }
          ],
          "defaultVolume": 100
        }
      ],
      "local_mix": [],
      "remote_mix": [],
      "aec_ref_mix": []
    },
    "usbCameraSourceInfo": [
      {
        "id": "046d_Logitech_Webcam_C930e_5F5DA2AE",
        "audioDevice": "alsa_input.usb-046d_Logitech_Webcam_C930e_5F5DA2AE-02.analog-stereo",
        "loopbackNode": "loopback_0",
        "sinkForLoopbackNode": "agc_snk_0",
        "latencyForLoopbackNode": null,
        "latencyForRecLoopbackNode": null,
        "agcNode": "agc_snk_0",
        "micMixNode": "mic_mix_0",
        "recLoopbackNode": null,
        "rtspAudioDevice": "mic_mix_0.monitor",
        "setRtspAudioDeviceAsDefaultSource": false
      },
      {
        "id": "00ABCDEF",
        "audioDevice": null,
        "loopbackNode": null,
        "sinkForLoopbackNode": null,
        "latencyForLoopbackNode": null,
        "latencyForRecLoopbackNode": null,
        "agcNode": null,
        "micMixNode": null,
        "recLoopbackNode": null,
        "rtspAudioDevice": null,
        "setRtspAudioDeviceAsDefaultSource": false
      }
    ],
    "auxInSourceInfo": [],
    "loopthroughSourceInfo": [
      {
        "id": "alsa_input.pci-0000_00_1b.0.analog-stereo",
        "audioDevice": "alsa_input.pci-0000_00_1b.0.analog-stereo",
        "loopbackNode": "loopback_loopthrough_0",
        "sinkForLoopbackNode": "main_mix",
        "latencyForLoopbackNode": "20",
        "latencyForRecLoopbackNode": null,
        "agcNode": null,
        "micMixNode": null,
        "recLoopbackNode": null,
        "rtspAudioDevice": null,
        "setRtspAudioDeviceAsDefaultSource": false
      }
    ],
    "compressorParms": "0,10,150,-18,5,5,3",
    "agcUsed": true,
    "micMixExists": false
  };

  const expectedAudioConfigBC = {
    "routing": [
      {
        "module": "module-combine-sink",
        "arguments": {
          "sink_name": "main_mix",
          "slaves": [
            "alsa_output.pci-0000_00_1b.0.analog-stereo"
          ]
        }
      },
      {
        "module": "module-loopback",
        "arguments": {
          "source": "alsa_input.pci-0000_00_1b.0.analog-stereo",
          "sink": "main_mix"
        }
      }
    ],
    "meetingId": "WHxq8NnfoiKfLTsqN",
    "currentDeviceRole": "broadcast_compositor",
    "streamRemoteAudio": true,
    "localMixNeeded": false,
    "remoteMixNeeded": false,
    "mainMixNeeded": true,
    "recordingMixNeeded": false,
    "calibrationState": "off",
    "defaultSource": "main_mix.monitor",
    "defaultSink": "main_mix",
    "contentSharingSink": "main_mix",
    "remoteSourceSink": "main_mix",
    "interTileMixSink": "main_mix",
    "sinkForReferenceAudioSample": {
      "teacher": "nullsink",
      "content": "main_mix",
      "background": "nullsink"
    },
    "rtspContentStreamSource": "main_mix.monitor",
    "sourcesWithRole": {
      "idle": [
        {
          "id": "alsa_input.usb-046d_Logitech_Webcam_C930e_5F5DA2AE-02.analog-stereo",
          "serial": "046d_Logitech_Webcam_C930e_5F5DA2AE",
          "activePort": {
            "id": "analog-input-mic",
            "title": "Microphone"
          },
          "device": "Webcam C930e Analog Stereo",
          "ports": [
            {
              "id": "analog-input-mic",
              "title": "Microphone"
            }
          ],
          "defaultVolume": 90
        }
      ],
      "aux_in": [
        {
          "id": "alsa_input.pci-0000_00_1b.0.analog-stereo",
          "activePort": {
            "id": "analog-input-internal-mic",
            "title": "Internal Microphone"
          },
          "device": "Built-in Audio Analog Stereo",
          "ports": [
            {
              "id": "analog-input-internal-mic",
              "title": "Internal Microphone"
            },
            {
              "id": "analog-input-mic",
              "title": "Microphone"
            },
            {
              "id": "analog-input-linein",
              "title": "Line In"
            }
          ],
          "defaultVolume": 10
        }
      ],
      "aec_mic_in": [],
      "aec_ref_in": []
    },
    "sinksWithRole": {
      "idle": [],
      "main_mix": [
        {
          "id": "alsa_output.pci-0000_00_1b.0.analog-stereo",
          "activePort": {
            "id": "analog-output-lineout",
            "title": "Line Out"
          },
          "device": "Built-in Audio Analog Stereo",
          "ports": [
            {
              "id": "analog-output-lineout",
              "title": "Line Out"
            },
            {
              "id": "analog-output-speaker",
              "title": "Speakers"
            }
          ],
          "defaultVolume": 100
        }
      ],
      "local_mix": [],
      "remote_mix": [],
      "aec_ref_mix": []
    },
    "usbCameraSourceInfo": [
      {
        "id": "046d_Logitech_Webcam_C930e_5F5DA2AE",
        "audioDevice": "alsa_input.usb-046d_Logitech_Webcam_C930e_5F5DA2AE-02.analog-stereo",
        "loopbackNode": null,
        "sinkForLoopbackNode": null,
        "latencyForLoopbackNode": null,
        "agcNode": null,
        "micMixNode": null,
        "recLoopbackNode": null,
        "latencyForRecLoopbackNode": null,
        "rtspAudioDevice": "alsa_input.usb-046d_Logitech_Webcam_C930e_5F5DA2AE-02.analog-stereo",
        "setRtspAudioDeviceAsDefaultSource": false
      },
      {
        "id": "00ABCDEF",
        "audioDevice": null,
        "loopbackNode": null,
        "sinkForLoopbackNode": null,
        "latencyForLoopbackNode": null,
        "agcNode": null,
        "micMixNode": null,
        "recLoopbackNode": null,
        "latencyForRecLoopbackNode": null,
        "rtspAudioDevice": null,
        "setRtspAudioDeviceAsDefaultSource": false
      }
    ],
    "auxInSourceInfo": [
      {
        "id": "alsa_input.pci-0000_00_1b.0.analog-stereo",
        "audioDevice": "alsa_input.pci-0000_00_1b.0.analog-stereo",
        "loopbackNode": "loopback_aux_0",
        "sinkForLoopbackNode": "main_mix",
        "latencyForLoopbackNode": null,
        "agcNode": null,
        "micMixNode": null,
        "recLoopbackNode": null,
        "latencyForRecLoopbackNode": null,
        "rtspAudioDevice": "main_mix.monitor",
        "setRtspAudioDeviceAsDefaultSource": true
      }
    ],
    "loopthroughSourceInfo": [],
    "compressorParms": "0,10,150,-18,5,5,3",
    "agcUsed": false,
    "micMixExists": false
  };

  let audioConfig = {};
  beforeEach(() => {
  });

  after(() => {});

  it('Should generate the right audio module configuration for overflow master role', () => {
    calculateAudioConfig(audioConfig, device, dummyLogger, false);
    assert.deepEqual(audioConfig, expectedAudioConfig);
  });
  it('Should generate the right audio module configuration for broadcast compositor role', () => {
    device.audioSourceConfig[0].role = 'aux_in';
    device.role = 'broadcast_compositor';
    device.streamRemoteAudio = true;
    device.usbCameraConfig[0].role = 'idle';
    calculateAudioConfig(audioConfig, device, dummyLogger, false);
    assert.deepEqual(audioConfig, expectedAudioConfigBC);
  });
});