import MeteorService from '../services/ddp/meteorService';

const entities = ['devices', 'meetings', 'sinks', 'compositions', 'layouts', 'sources', 'institutes'];
const actions = ['added', 'changed', 'removed'];

class SingleDeviceMeteorService extends MeteorService {
  constructor({
    id,
    meteorClient,
    device,
  }) {
    super({
      id: id || device._id,
      meteorClient,
    });

    this._device = device;
  }

  get device() {
    return this._device;
  }

  set device(device) {
    this._device = device;
  }

  get meteorClientListeners() {
    if (!this._meteorListeners) {
      this._meteorListeners = new Map();

      for (const entity of entities) {
        for (const action of actions) {
          switch (entity) {
          case 'meetings':
          case 'institutes':
            this._meteorListeners.set(`${entity}:${action}`, this._trigger.bind(this, entity, action));
            break;
          case 'devices':
            this._meteorListeners.set(`${entity}:${action}`, (device, ...params) => {
              if (device._id !== this._device._id) {
                this._trigger(entity, action, device, ...params);
              }
            });
            break;
          default:
            this._meteorListeners.set(`DeviceData:${this._device._id}:${entity}:${action}`, this._trigger.bind(this, entity, action));
          }
        }
      }

      this._meteorListeners.set(`${this._device._id}:removed`, (...params) => this.scheduler.schedule('_onThisDeviceRemoved', ...params));
      this._meteorListeners.set(`${this._device._id}:changed`, (...params) => this.scheduler.schedule('_onThisDeviceChanged', ...params));

      this._meteorListeners.set(`${this._device._id}:rpc`, async (rpc) => {
        if (this.supportedRPCCommands.includes(rpc.command) && this.applyActions) {
          await this._triggerRPCCommand(rpc);
        }
      });
    }
    return this._meteorListeners;
  }

  async _onDestroy() {
    await super._onDestroy();
    delete this._device;
  }
}

export default SingleDeviceMeteorService;
