/* eslint-disable */
import * as EJSON from 'ddp-ejson';

class IdMap extends Map{
  constructor (idStringify, idParse) {
    super();
    this._idStringify = idStringify || JSON.stringify
    this._idParse = idParse || JSON.parse
  }

  // Some of these methods are designed to match methods on OrderedDict, since
  // (eg) ObserveMultiplex and _CachingChangeObserver use them interchangeably.
  // (Conceivably, this should be replaced with "UnorderedDict" with a specific
  // set of methods that overlap between the two.)
  set(id,value){
    return super.set(this._idStringify(id),value);
  }

  get(id){
    return super.get(this._idStringify(id))
  }

  has(id){
    return super.has(this._idStringify(id));
  }

  delete(id){
    return super.delete(this._idStringify(id))
  }

  remove (id) {
    return this.delete(id);
  }

  empty () {
    return this.size === 0
  }

  // Iterates over the items in the map. Return `false` to break the loop.
  forEach (iterator) {
    const keys = this.keys()
    for (const key of keys) {

      const breakIfFalse = iterator.call(null, super.get(key), this._idParse(key))
      if (breakIfFalse === false){
         return
      }
    }
  }

  setDefault (id, def) {
    const key = this._idStringify(id)
    if (super.has(key))
      return this.get(key)
    super.set(key,def);
    return def
  }
  
  // Assumes that values are EJSON-cloneable, and that we don't need to clone
  // IDs (ie, that nobody is going to mutate an ObjectId).
  clone () {
    const clone = new IdMap(this._idStringify, this._idParse)
    this.forEach(function (value, id) {
      clone.set(id, EJSON.clone(value))
    })
    return clone
  }
}


export default IdMap;