/* eslint-disable */
/**
 * Generate Mongo Identifiers.
 * based off https://github.com/meteor/meteor/blob/devel/packages/random/random.js
 */


import * as nodeCrypto from 'crypto';


const UNMISTAKABLE_CHARS = "23456789ABCDEFGHJKLMNPQRSTWXYZabcdefghijkmnopqrstuvwxyz";
const BASE64_CHARS = "abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ" +
  "0123456789-_";

class RandomGenerator{
  constructor(){}

  fraction() {
    return parseInt(this.hexString(8), 16) * 2.3283064365386963e-10; // 2^-32
  }

  hexString(digits) {

    const numBytes = Math.ceil(digits / 2);
    var bytes;
    // Try to get cryptographically strong randomness. Fall back to
    // non-cryptographically strong if not available.
    try {
      bytes = nodeCrypto.randomBytes(numBytes);
    } catch (e) {
      // XXX should re-throw any error except insufficient entropy
      bytes = nodeCrypto.pseudoRandomBytes(numBytes);
    }
    // If the number of digits is odd, we'll have generated an extra 4 bits
    // of randomness, so we need to trim the last digit.
    return bytes.toString("hex").substring(0, digits);
  }

  _randomString(charsCount,alphabet) {
    const digits = [];
    for (var i = 0; i < charsCount; i++) {
      digits[i] = this.choice(alphabet);
    }
    return digits.join("");
  }

  id(charsCount = 17) {
    // 17 characters is around 96 bits of entropy, which is the amount of
    // state in the Alea PRNG.
    return this._randomString(charsCount, UNMISTAKABLE_CHARS);
  }

  secret(charsCount = 43) {
    // Default to 256 bits of entropy, or 43 characters at 6 bits per
    // character.
    return this._randomString(charsCount, BASE64_CHARS);
  }

  choice(arrayOrString) {
    var index = Math.floor(this.fraction() * arrayOrString.length);
    if (typeof arrayOrString === "string")
      return arrayOrString.substr(index, 1);
    else
      return arrayOrString[index];
  }
}

export default new RandomGenerator();