/* eslint-disable */
// An even simpler queue of tasks than the fiber-enabled one.  This one just
// runs all the tasks when you call runTask or flush, synchronously.
//
class SynchronousQueue {
  constructor () {
    this._tasks = []
    this._running = false
    this._runTimeout = null
  }

  runTask (task) {
    if (!this.safeToRunTask())
      throw new Error('Could not synchronously run a task from a running task')
    this._tasks.push(task)
    const tasks = this._tasks
    this._tasks = []
    this._running = true

    if (this._runTimeout) {
      // Since we're going to drain the queue, we can forget about the timeout
      // which tries to run it.  (But if one of our tasks queues something else,
      // the timeout will be correctly re-created.)
      clearTimeout(this._runTimeout)
      this._runTimeout = null
    }

    try {
      while (tasks.length !== 0) {
        const t = tasks.shift()
        try {
          t()
        } catch (e) {
          if (tasks.length === 0) {
            // this was the last task, that is, the one we're calling runTask
            // for.
            throw e
          }
        }
      }
    } finally {
      this._running = false
    }
  }

  queueTask (task) {
    this._tasks.push(task)
    // Intentionally not using Meteor.setTimeout, because it doesn't like runing
    // in stubs for now.
    if (!this._runTimeout) {
      this._runTimeout = setTimeout(this.flush.bind(this), 0)
    }
  }

  flush () {
    this.runTask(function () {})
  }

  drain () {
    if (!this.safeToRunTask())
      return
    while (this._tasks.length !== 0) {
      this.flush()
    }
  }

  safeToRunTask () {
    return !this._running
  }

}

export default SynchronousQueue
