/*
eslint-disable no-underscore-dangle
*/

import ConnectionMonitor from '../connectionMonitor';

/**
 * Connection monitor for DDP connections
 */
class DDPConnectionMonitor extends ConnectionMonitor {
  constructor(params) {
    super(params);
    if (!params.ddpClient) {
      throw new Error('DDP Connection monitor needs a DDP connection');
    }

    this._ddpClient = params.ddpClient;
    this._ddpClient.on('message', this.messageReceived.bind(this));

    // Timeout is called when heartbeat timeout is fired
    this.on('timeout', () => {
      this._ddpClient.socket._stream.destroy(); // We destroy the raw socket, since closing the websocket sometimes hangs, see: https://github.com/faye/faye-websocket-node/issues/56
    });
  }

  _logHeartbeatTimeout() {
    this._log('error', `Heartbeat timeout reached while being connected with ${this._ddpClient.host}`, { serviceLog: true });
  }

  _sendPing() {
    this._log('debug', 'Pinging server');
    this._ddpClient._send({ msg: 'ping' });
  }

  destroy() {
    super.destroy();
    delete this._ddpClient;
  }
}

export default DDPConnectionMonitor;
