const deviceStatus = {
  UNKNOWN: 0,
  OFFLINE: 1,
  WARNING: 2,
  ONLINE: 3,
};
Object.freeze(deviceStatus);

const userRoleInMeeting = {
  PARTICIPANT: 1,
  HOST: 2,
  OPERATOR: 3,
};
Object.freeze(userRoleInMeeting);

const equipmentState = {
  UNCLAIMED: 1,
  CLAIMING: 2,
  WAITING: 3,
  CLAIMED: 4,
};
Object.freeze(equipmentState);

const showTitlePolicy = {
  NEVER: 0,
  FADE: 1,
  ALWAYS: 2,
};
Object.freeze(showTitlePolicy);

const state = {
  NOT_IN_MEETING: 'notInMeeting',
  FALLBACK: 'fallback',
  CLONE_POD: 'clone_pod',
  CLONE_MASTER: 'clone_master',
  LOCKED: 'locked',
  DEFAULT: 'default',
};
Object.freeze(state);

const usbCameraSettingType = {
  MENU: 'menu',
  INT: 'int',
  BOOL: 'bool',
};
Object.freeze(usbCameraSettingType);

const recordingState = {
  NOT_RECORDING: 0,
  RECORDING: 1,
  PAUSED: 2,
};
Object.freeze(recordingState);

const commands = {
  REBOOT: 'reboot',
  IDENTIFY: 'showIdentificationMessage',
  IDENTIFY_SCREENS: 'id_screens',
  DISCONNECT_BYOD_CLIENTS: 'disconnectBYODClients',
  DISCONNECT_PREVIEW_CLIENTS: 'disconnectPreviewClients',
  FORCE_UPGRADE: 'forceUpgrade',
  SET_TELEPORT: 'setTeleport',
  PLAY_REFERENCE_SAMPLE: 'playReferenceSample',
  STOP_ALL_REFERENCE_SAMPLES: 'stopAllReferenceSamples',
  FORCE_PA_DETECT_INPUTS_OUTPUTS: 'force_pa_detect_inputs_outputs',
  UNPAIR: 'unpair',
  CLEAR_TELEPORT_CACHE: 'clearTeleportCache',
  RESET_HW_CLOCK: 'resetHWClock',
  VERIFY_RECORDING_OPTIONS: 'verifyRecordingOptions',
};
Object.freeze(commands);

const compositionType = {
  CINERGY: 'cinergy',
  BROWSER: 'browser',
};
Object.freeze(compositionType);

const sinkType = {
  MJPEG_STREAM: 'mjpeg_stream',
  RTSP_STREAM: 'rtsp_stream',
  RTSP_AUDIO_STREAM: 'rtsp_audio_stream', // audio only rtsp stream
  RECORD_STREAM: 'record_stream', // lecture capture
  SWFG_STREAM: 'swfg_stream', // software framegrabber
  SCREEN: 'screen',
};
Object.freeze(sinkType);

/**
 * ENUM for who added the source
 */
const sourceAddedBy = {
  SYSTEM: 0,
  DEVICE: 1,
  USER: 2,
};
Object.freeze(sourceAddedBy);

/**
 * ENUM for the different types of sources
 * @type {{STREAM: string, WALLPAPER: string, BYOD: string, FILE: string, ACTIVITY: string, OTHER: string}}
 */
const sourceType = {
  STREAM: 'stream',
  WALLPAPER: 'wallpaper',
  USB: 'usb',
  BYOD: 'byod',
  FILE: 'file',
  ACTIVITY: 'activity',
  OTHER: 'other',
};
Object.freeze(sourceType);

const audioSinkRole = {
  IDLE: 'idle',
  MAIN_MIX: 'main_mix',
  LOCAL_MIX: 'local_mix',
  REMOTE_MIX: 'remote_mix',
  AEC_REF_MIX: 'aec_ref_mix',
};
Object.freeze(audioSinkRole);

const pulseaudioModules = {
  COMBINED_SINK: 'module-combine-sink',
  NULL_SINK: 'module-null-sink',
  LOOPBACK: 'module-loopback',
  TUNNEL_SINK_NEW: 'module-tunnel-sink-new',
  LADSPA_SINK: 'module-ladspa-sink',
};
Object.freeze(pulseaudioModules);

const audioModuleRole = {
  MAIN_MIX: 'main_mix',
  LOCAL_MIX: 'local_mix',
  REMOTE_MIX: 'remote_mix',
  MIC_MIX: 'mic_mix',
  RECORDING_MIX: 'recording_mix',
  LOOPBACK: 'loopback',
  TUNNEL_SINK: 'tnl_snk', // tunnel sinks (inter-tile mix) have this string in their name
  AGC_SINK: 'agc_snk', // automatic gain control module (= 'compressor/expander')
};
Object.freeze(audioModuleRole);

const audioCalibrationState = {
  OFF: 'off', // Audio calibration is not switched on at this moment
  CONTENT_SHARING: 'content_sharing', // Calibrating content sharing volume
  MICROPHONE: 'microphone', // Calibrating microphone volume
};
Object.freeze(audioCalibrationState);

const audioDefaultParameters = {
  maxVolume: 65537,
};

Object.freeze(audioDefaultParameters);

const audioSourceRole = {
  IDLE: 'idle',
  AUX_IN: 'aux_in',
  AEC_MIC_IN: 'aec_mic_in',
  AEC_REF_IN: 'aec_ref_in',
};
Object.freeze(audioSourceRole);

const genericStreamNames = {
  CAM_VIEW: 'cam-self-view',
  TILE_CAMERA: 'tile-webcam-view',
  MANAGED_CAMERA: 'managed-webcam-view',
  NRC_CONTENT: 'nrc-content',
  AUDIO_MIX: 'audio-mix',
};
Object.freeze(genericStreamNames);

const audioReferenceSampleType = {
  TEACHER: 'teacher', // simulate teacher speaking
  CONTENT: 'content', // simulate sharing content with audio
  BACKGROUND: 'background', // simulate background noise
};
Object.freeze(audioReferenceSampleType);

const usbCameraRole = {
  IDLE: 'idle',
  ROOM: 'room',
  PRESENTER: 'presenter',
  CONTENT: 'content',
  OTHER: 'other',
};
Object.freeze(usbCameraRole);

const recordingSourceType = {
  PRIMARY: 'Primary',
  SECONDARY: 'Secondary',
  AUDIO: 'Audio',
};
Object.freeze(recordingSourceType);

const deviceRole = {
  IDLE: 'idle',
  MASTER: 'master',
  COLLABORATION_MASTER: 'collaboration_master',
  COLLABORATION_OVERFLOW_MASTER: 'collaboration_overflow_master',
  COLLABORATION_POD: 'collaboration_pod',
  OVERFLOW_MASTER: 'overflow_master',
  OVERFLOW_SLAVE: 'overflow_slave',
  VIRTUAL_MASTER: 'virtual_master',
  VIRTUAL_OFFWALL_TILE: 'virtual_offwall_tile',
  VIRTUAL_TILE: 'virtual_tile',
  VIRTUAL_CAMERA_MANAGER: 'virtual_camera_manager',
  CAMERA_MANAGER_MASTER: 'camera_manager_master',
  CAMERA_MANAGER_SLAVE: 'camera_manager_slave',
  FAR_END: 'far_end',
  CONFIDENCE_OVERFLOW_MASTER: 'confidence_overflow_master',
  CONFIDENCE_OVERFLOW_SLAVE: 'confidence_overflow_slave',
  COLLABORATION_CONFIDENCE_MASTER: 'collaboration_confidence_master',
  AUXILIARY: 'auxiliary',
  BROADCAST_COMPOSITOR: 'broadcast_compositor',
  GALLERY_VIEW_COMPOSITOR: 'gallery_view_compositor',
};
Object.freeze(deviceRole);

const deviceRoleLabels = {};
deviceRoleLabels[deviceRole.IDLE] = {
  label: 'Not Used',
  extendedLabel: 'Not Used',
};
deviceRoleLabels[deviceRole.MASTER] = {
  label: 'Main Node',
  extendedLabel: 'Main Node',
};
deviceRoleLabels[deviceRole.AUXILIARY] = {
  label: 'Auxiliary Node',
  extendedLabel: 'Auxiliary Node',
};
deviceRoleLabels[deviceRole.COLLABORATION_MASTER] = {
  label: 'Main Node',
  extendedLabel: 'Collaboration Main Node',
};
deviceRoleLabels[deviceRole.COLLABORATION_OVERFLOW_MASTER] = {
  ...deviceRoleLabels[deviceRole.COLLABORATION_MASTER],
  extendedLabel: 'Collaboration Main Node - Collaboration Breakout room',
};
deviceRoleLabels[deviceRole.COLLABORATION_CONFIDENCE_MASTER] = {
  label: 'Far-View/Confidence Node',
  extendedLabel: 'Far-View/Confidence Node - Collaboration room',
  description: 'The selected device drives a Far-View or Confidence Display',
};
deviceRoleLabels[deviceRole.COLLABORATION_POD] = {
  label: 'Collaboration Group Node',
  extendedLabel: 'Collaboration Group Node',
};
deviceRoleLabels[deviceRole.OVERFLOW_MASTER] = {
  label: 'Main Node',
  extendedLabel: 'Main Node - Main Overflow room',
  description: 'The selected device is driving the Main Display(s) in the classroom or auditorium.',
};
deviceRoleLabels[deviceRole.OVERFLOW_SLAVE] = {
  label: 'Main Node',
  extendedLabel: 'Main Node - Overflow room',
};
deviceRoleLabels[deviceRole.VIRTUAL_MASTER] = {
  label: 'Main Node',
  extendedLabel: 'Main Node - Virtual room',
};
deviceRoleLabels[deviceRole.VIRTUAL_OFFWALL_TILE] = {
  label: 'Display OffWall Tile Node',
  extendedLabel: 'Display OffWall Tile Node',
};
deviceRoleLabels[deviceRole.VIRTUAL_TILE] = {
  label: 'Display Wall Tile Node',
  extendedLabel: 'Display Wall Tile Node',
};
deviceRoleLabels[deviceRole.VIRTUAL_CAMERA_MANAGER] = {
  label: 'Camera Manager Node',
  extendedLabel: 'Camera Manager Node - Virtual room',
};
deviceRoleLabels[deviceRole.CAMERA_MANAGER_MASTER] = {
  label: 'Camera Manager Node',
  extendedLabel: 'Camera Manager Node',
};
deviceRoleLabels[deviceRole.CAMERA_MANAGER_SLAVE] = {
  ...deviceRoleLabels[deviceRole.CAMERA_MANAGER_MASTER],
  extendedLabel: 'Camera Manager Node - Overflow room',
};
deviceRoleLabels[deviceRole.CONFIDENCE_OVERFLOW_MASTER] = {
  label: 'Far-View/Confidence Node',
  extendedLabel: 'Far-View/Confidence Node - Overflow Main room',
};
deviceRoleLabels[deviceRole.CONFIDENCE_OVERFLOW_SLAVE] = {
  label: 'Far-View/Confidence Node',
  extendedLabel: 'Far-View/Confidence Node - overflow slave',
};
deviceRoleLabels[deviceRole.FAR_END] = {
  label: 'Far-View/Confidence Node',
  extendedLabel: 'Far-View/Confidence Node',
};
deviceRoleLabels[deviceRole.BROADCAST_COMPOSITOR] = {
  label: 'Broadcast Compositor Node',
  extendedLabel: 'Broadcast Compositor Node',
};
deviceRoleLabels[deviceRole.GALLERY_VIEW_COMPOSITOR] = {
  label: 'Gallery View Compositor Node',
  extendedLabel: 'Gallery View Compositor Node',
};
Object.freeze(deviceRoleLabels);

const displayRole = {
  IDLE: 'idle',
  MAIN_OUT: 'main_out',
  LOCAL_CLONE: 'local_clone',
  REMOTE_CLONE: 'remote_clone',
  SELF_VIEW_SLAVE: 'self_view_slave',
  SELF_VIEW_MASTER_COMBINED: 'self_view_master_combined',
  SELF_VIEW_MASTER_CONTENT: 'self_view_master_content',
  SELF_VIEW_MASTER_TEACHER: 'self_view_master_teacher',
  SLAVE_COMBINED_FROM_MASTER: 'slave_combined_from_master',
  SLAVE_CONTENT_FROM_MASTER: 'slave_content_from_master',
  SLAVE_TEACHER_FROM_MASTER: 'slave_teacher_from_master',
  COLLABORATION_MASTER_MAIN_OUT: 'collaboration_master_main_out',
  COLLABORATION_POD_MAIN_OUT: 'collaboration_pod_main_out',
  OVERFLOW_REMOTE: 'overflow_remote',
  HTML_TILE_OUT: 'html_tile_out',
  HTML_OFFWALL_TILE_OUT: 'html_offwall_tile_out',
  HTML_CAMERA_MANAGER: 'html_camera_manager',
  HTML_GUI_OUT: 'html_gui_out',
  HTML_WHITEBOARD_OUT: 'html_whiteboard_out',
  RTSP_CAMERA_MANAGER: 'rtsp_camera_manager',
  BROADCAST_COMPOSITOR: 'broadcast_compositor',
  GALLERY_VIEW: 'gallery_view',
  AUXILIARY_MAIN: 'auxiliary_main',
  AUXILIARY: 'auxiliary',
};
Object.freeze(displayRole);

export {
  sourceAddedBy,
  showTitlePolicy,
  sourceType,
  commands,
  sinkType,
  deviceStatus,
  deviceRole,
  deviceRoleLabels,
  displayRole,
  state,
  recordingState,
  compositionType,
  audioSinkRole,
  audioSourceRole,
  usbCameraSettingType,
  audioModuleRole,
  audioDefaultParameters,
  pulseaudioModules,
  genericStreamNames,
  equipmentState,
  audioReferenceSampleType,
  audioCalibrationState,
  usbCameraRole,
  userRoleInMeeting,
  recordingSourceType,
};
