import PurpleEventEmitter from '../utils/eventEmitter';

class EventProxy extends PurpleEventEmitter {
  constructor() {
    super();
    this.contextMap = new Map();
  }

  register(context, events) {
    if (!this.contextMap.has(context)) {
      this.contextMap.set(context, {});
    }
    const listeners = this.contextMap.get(context);

    events
      .filter(eventName => !listeners.hasOwnProperty(eventName))
      .forEach((eventName) => {
        listeners[eventName] = (...data) => {
          this.emit(`${context.serviceName}:${eventName}`, ...data);
        };
        context.on(eventName, listeners[eventName]);
      });
  }

  unregister(context) {
    const listeners = this.contextMap.get(context);
    if (listeners) {
      Object.keys(listeners).forEach((eventName) => {
        context.removeListener(eventName, listeners[eventName]);
      });
      this.contextMap.delete(context);
    }
  }
}

export default EventProxy;
export { EventProxy };
