export default class IntervalRunner {
  constructor(callback, desiredFrequency, maxIterations = Infinity) {
    this.timeoutId = null;
    this.running = false;
    this.iteration = 1;
    this.desiredFrequency = desiredFrequency;
    this.maxIterations = maxIterations;
    this.callback = callback;
    this.destroyed = false;
    this.cycle = this.cycle.bind(this);
  }

  set desiredFrequency(desiredFrequency) {
    if (this._desiredFrequency === desiredFrequency) {
      return;
    }

    this._desiredFrequency = desiredFrequency;
    this.timeBetween = 1000 / this._desiredFrequency;

    if (this.running) {
      clearTimeout(this.timeoutId);
      setTimeout(this.cycle, this.timeTillNextExecution);
    }
  }

  get desiredFrequency() {
    return this._desiredFrequency;
  }

  get timeTillNextExecution() {
    const timePassedSinceLastScheduled = new Date() - this._lastScheduledAt;
    return Math.max(0, this.timeBetween - timePassedSinceLastScheduled);
  }

  cycle(force = false) {
    if (!force && (!this.running || this.iteration > this.maxIterations)) {
      return;
    }

    this.iteration += 1;
    if (this.running) {
      this._lastScheduledAt = new Date();
      clearTimeout(this.timeoutId);
      this.timeoutId = setTimeout(this.cycle, this.timeTillNextExecution);
    }

    this.callback();
  }

  start() {
    if (this.running) {
      return;
    }

    if (this.destroyed) {
      throw Error('IntervalRunner has been destroyed');
    }

    this.running = true;
    this._lastScheduledAt = new Date();
    setTimeout(this.cycle, this.timeBetween);
  }

  stop() {
    if (!this.running) {
      return;
    }
    clearTimeout(this.timeoutId);
    this.running = false;
  }

  destroy() {
    this.stop();
    delete this.callback;
    this.destroyed = true;
  }
}
