import { Service } from './service';
import createUI from './ui';

/**
 * The messenger class shows messages on all available displays.
 * This can be used to show e.g. startup, error and identification messages
 */
class Messenger extends Service {
  constructor(options = {}) {
    super(options);
  }

  /**
   * Starts all children
   */
  async _onStart() {
    this._ui = createUI({ logger: this._logger });
    Object.entries(this._ui).forEach(([name, fn]) => {
      this[name] = fn;
    });
    return super._onStart();
  }

  /**
   * Hides all messages
   */
  async _onStop() {
    if (this._ui) {
      this._ui.destroy();
      Object.keys(this._ui).forEach((name) => {
        delete this[name];
      });
      delete this._ui;
    }
    return super._onStop();
  }
}

export default Messenger;
