import { promises as fs } from 'fs';
import { MESSAGE, ROLE, TEST } from '@barco/network-test';
import Test from '@barco/network-test/dist/tests/Test';

import { VERSION_FILE } from '../../files';
import { compareVersions } from '../upgrades/systemVersionMonitor';

export default class SystemVersionTest extends Test {
  async start() {
    let currentVersion;
    try {
      currentVersion = await fs.readFile(VERSION_FILE, 'utf8').trim();
    } catch (error) {
      currentVersion = this.context.self.version;
    }

    if (compareVersions(this.context.latestDeviceVersion, currentVersion) > 0) {
      throw new Test.Error(MESSAGE.OUTDATED, {
        current: currentVersion,
        latest: this.context.latestDeviceVersion,
      });
    }
  }
}

SystemVersionTest.ForTest = TEST.CHECK_VERSION;
SystemVersionTest.ForRoles = [ROLE.SLAVE];
