/**
 * (C) 2015 Copyright BARCO NV
 * All rights reserved. Reproduction in whole or in part is
 * prohibited without the written consent of the copyright owner.
 *
 * Part of Barco WeConnect
 */
import Service from '../service';

/**
 * This class represents a client connection to the MJPEG server.
 */
class PreviewHandler extends Service {
  /**
   * @param logger
   * @param id - Identifier, useful for logging and debugging
   * @param req - expressjs request object
   * @param res - expressjs response object
   */
  constructor({ id, req, res }) {
    super({ id });

    const params = { id, req, res };
    Object.keys(params).forEach((param) => {
      if (params[param] === undefined) throw new Error(`Please provide a valid ${param}`);
      this[`_${param}`] = params[param];
    });

    this._log('debug', 'PreviewHandler created.');

    this._req.socket.on('close', () => {
      if (this.isActive) {
        this._log('debug', 'Client request to close.');
        this.close();
      }
    });
  }

  get id() {
    return this._id;
  }

  _sendInitialHeaders() {
    // Send initial MJPEG headers
    this._res.writeHead(200, {
      'Content-Type': 'multipart/x-mixed-replace;boundary=--ThisRandomString',
      Connection: 'keep-alive',
      'Cache-Control': 'no-cache, no-store, must-revalidate',
      Pragma: 'no-cache',
      Expires: '0',
    });
    this._res.write('--ThisRandomString\r\n');
  }

  /**
   * Send next MJPEG preview to the MJPEG stream
   * @param jpeg
   */
  sendPreviewPart(jpeg) {
    if (this.isActive) {
      this._res.write(jpeg, 'binary');
    }
  }

  _onStart() {
    this._sendInitialHeaders();
    return super._onStart();
  }

  /**
   * Close connection with client.
   */
  close() {
    this.destroy();
  }

  _onDestroy() {
    this._res.end();
    delete this._res;
    delete this._req;
    return super._onDestroy();
  }
}
export default PreviewHandler;
