import { isString, isFunction } from 'lodash';

import ServiceMixin from '../serviceMixin';

class Scheduler extends ServiceMixin {
  constructor(service) {
    super(service);
    this._service.scheduler = this;
  }

  flush() {
    return this._doSchedule(() => Promise.resolve());
  }

  schedule(method, ...params) {
    if (this._shouldSchedule(method, ...params)) {
      if (isString(method)) {
        if (!method.includes('Device') || method.includes('Added') || method.includes('Removed')) {
          this.log('verbose', `Scheduling ${method} with params: ${JSON.stringify(params, null, 2)}`);
        }
        const methodName = method;
        method = () => this._service[methodName](...params);
      } else {
        this.log('silly', `Scheduling ${method.name || 'unnamed'} function`);
      }

      return this._doSchedule(method);
    }
    // XXX: should maybe reject?
    return Promise.resolve();
  }

  _shouldSchedule(method, ...params) { //eslint-disable-line
    return (
      (!('applyActions' in this._service) || this._service.applyActions)
      && ((
        isString(method) && method in this._service
      ) || (
        isFunction(method)
      ))
    );
  }

  _doSchedule() {
    throw new Error('Scheduler should implement _doSchedule method');
  }

  destroy() {
    delete this._service.scheduler;
    super.destroy();
  }
}

export default Scheduler;
