import BaseScheduler from './scheduler';

class SynchronousScheduler extends BaseScheduler {
  constructor(...params) {
    super(...params);
    this._executionQueue = [];
  }

  stop() {
    this._executionQueue.forEach((queueObject) => {
      queueObject.reject(new Error('Rejected because scheduler has stopped'));
    });

    this._executionQueue = [];
    return super.stop();
  }

  _doSchedule(method) {
    return new Promise((resolve, reject) => {
      this._executionQueue.push({
        resolve,
        reject,
        method,
      });

      if (this._executionQueue.length === 1) {
        this._processQueue();
      }
    });
  }

  async _processQueue() {
    if (this._executionQueue.length > 0) {
      const queueObject = this._executionQueue[0];
      try {
        const result = await queueObject.method();
        queueObject.resolve(result);
      } catch (error) {
        queueObject.reject(error);
      } finally {
        this._executionQueue.shift();
        this._processQueue();
      }
    }
  }
}

export default SynchronousScheduler;
