import * as Boom from 'boom';

import { wrapAsync } from '../server';
import ServiceMixin from '../serviceMixin';
import decodeToken from '../../utils/jwt';

/**
 * HTTP endpoints to receive the initial configuration.
 * Also provides some other utilities for the pairing tool, like identification messages.
 */
class IdentifyMixin extends ServiceMixin {
  init() {
    // Show identification message (useful for pairing)
    this._service.addRoute('/showMessage', 'post', wrapAsync(this.showIdentificationMessage.bind(this)));
  }

  /**
   * Shows an identification message
   * @param {*} req
   * @param {*} res
   */
  async showIdentificationMessage(req, res) {
    let title;

    try {
      ({ title } = await decodeToken(req.body.token));
    } catch (error) {
      this._log('error', 'Error while decoding message token', error);
      throw Boom.badRequest('Invalid request');
    }

    res.json(true);

    try {
      this._log('info', `Received show identify message command. title = ${title}`);
      await this._service._showMessage({
        text: title || 'identifying device',
        time: 25,
      });
      this._log('info', 'Done showing identify message');
    } catch (error) {
      this._log('error', 'Error during show identify message command');
    }
  }
}

export default IdentifyMixin;
