import * as pify from 'pify';
import ServiceMixin from '../serviceMixin';
import { createServer } from '../server';

const emptyGifBuffer = Buffer.from('R0lGODlhAQABAPAAAAAAAAAAACH5BAEAAAAALAAAAAABAAEAAAICRAEAOw==', 'base64');
const emptyGifBufferLength = emptyGifBuffer.length;

/**
 * HTTP Endpoints to check device presence, can be used to figure out if user is within the network of the device
 */
class PingMixin extends ServiceMixin {
  init() {
    this._service.app.get('/imgping', this.imgping.bind(this));
    // Quick check if server is active, useful for pairing tool
    this._service.addRoute('/check', 'post', this.check.bind(this));
    this._service.addRoute('/check', 'get', this.check.bind(this));
  }

  /**
   * Ping-pong, Just to check if pairingserver is reachable
   */
  check(req, res) {
    res.json(true);
  }

  /**
   * Serves an empty gif, as a fallback for https to http requests
   */
  imgping(req, res) {
    res.writeHead(200, {
      'Content-Type': 'image/gif',
      'Content-Length': emptyGifBufferLength,
    });
    res.end(emptyGifBuffer);
  }

  // XXX: Ping server used to run on 8092, but is now merged into pairing server
  // This should be here until the cloud uses 8090
  async start() {
    this.server = await createServer(this._service.app, 8092);
  }

  async stop() {
    // Ping server used to run on 8092, but is now merged into pairing server
    // This should be here until the cloud uses 8090
    if (this.server) {
      await pify(this.server.close.bind(this.server))();
      delete this.server;
    }
  }
}

export default PingMixin;
