/**
 * Created by karolis on 6/21/17.
 */
import * as fs from 'fs-extra';
import * as path from 'path';
import * as Boom from 'boom';

import { PreviewProvider } from '../previews/provider';
import { SHARED_MEMORY_DIR } from '../../files';

class SnapshotProvider extends PreviewProvider {
  constructor(params) {
    super(params);
    this.composition = params.composition;
  }

  get sourcePath() {
    return path.join(SHARED_MEMORY_DIR, `${this.composition}.jpeg`);
  }

  get readPath() {
    return path.join(SHARED_MEMORY_DIR, `${this.composition}_to_read.jpeg`);
  }

  get lastJPG() {
    try {
      fs.moveSync(this.sourcePath, this.readPath, { overwrite: true });
      this.previewDataJPG = fs.readFileSync(this.readPath);
      this._log('silly', 'Serving new snapshot');
    } catch (err) {
      this._log('debug', 'New snapshot file is not ready yet. Serving the old one');
    }
    return this._previewData.dataJPG;
  }

  /**
   * Handle an incoming request. This means serving the last snapshot
   * @param req
   * @param res
   */
  handleIncomingRequest(req, res) {
    this._log('silly', 'Serving last preview for path');
    if (super.handleIncomingRequest(req, res)) {
      this._serveLastSnapshot(req, res);
    }
  }

  /**
   * Serve the last snapshot if available, otherwise return 500
   * @param req
   * @param res
   * @private
   */
  _serveLastSnapshot(req, res) {
    const jpg = this.lastJPG;
    if (!jpg || !jpg.length) {
      this._log('verbose', 'jpg is not set. Returning 404');
      throw Boom.notFound();
    }

    let filename = req.query.filename || 'snapshot';
    filename += '.jpg';
    const headers = {
      'Content-Disposition': `attachment; filename=${filename}`,
      'Content-Length': jpg.length,
    };

    res.writeHead(200, headers);
    res.end(jpg);
  }

  _removeCachedJPGFiles() {
    try {
      fs.removeSync(this.sourcePath);
      fs.removeSync(this.readPath);
    } catch (error) {}
  }

  _onStart() {
    // remove cached jpeg files to make sure we don't serve a file from the previous lecture!
    this._removeCachedJPGFiles();
    return super._onStart();
  }
}

export default SnapshotProvider;
