/* eslint-env browser */

function startProgress() {
  document.querySelector('.loader-logo').classList.add('loader-logo--animated');
}

function stopProgress() {
  const stopAnimation = (e) => {
    document.querySelector('.loader-logo').classList.remove('loader-logo--animated');
    e.target.removeEventListener('animationiteration', stopAnimation);
  };
  const animatedDot = document.querySelector('.loader-logo.loader-logo--animated .loader-logo__dot--top-left');
  if (animatedDot) {
    animatedDot.addEventListener('animationiteration', stopAnimation);
  }
}

function addMessage(id, { text }) {
  const message = document.createElement('p');
  message.classList.add('message', 'message--hidden');
  message.appendChild(document.createTextNode(text));
  message.dataset.messageId = id;
  message.dataset.messageTime = Date.now();
  document.querySelector('.messages').insertBefore(message, document.querySelector('.messages').firstElementChild);
  setTimeout(() => {
    message.classList.remove('message--hidden');
  }, 500);
}

function removeMessage(id) {
  const message = document.querySelector(`.message[data-message-id="${id}"]`);
  if (message) {
    // + 2000, because we want a message to be visible for at least 1 second
    // but animations to fade in already take about a second
    const minEndTime = Number.parseInt(message.dataset.messageTime, 10) + 2000;
    const timeLeft = Math.max(minEndTime - Date.now(), 0);
    setTimeout(() => {
      message.classList.add('message--hidden', 'message--old');
      setTimeout(() => {
        message.remove();
      }, 1000);
    }, timeLeft);
  }
}

function setInfo(key, value) {
  const infoItem = document.querySelector(`.info-item[data-info-item="${key}"]`);
  if (value) {
    infoItem.classList.remove('info-item--unset');
    infoItem.querySelector('.info-item__value').textContent = `${value}`;
  } else {
    infoItem.classList.add('info-item--unset');
    infoItem.querySelector('.info-item__value').textContent = '';
  }
}

const { ipcRenderer } = require('electron');

ipcRenderer.send('ready');
ipcRenderer.on('info', (_event, key, value) => {
  setInfo(key, value);
});
ipcRenderer.on('addMessage', (_event, id, message) => {
  addMessage(id, message);
});
ipcRenderer.on('removeMessage', (_event, id) => {
  removeMessage(id);
});
ipcRenderer.on('startProgress', (_event) => {
  startProgress();
});
ipcRenderer.on('stopProgress', (_event) => {
  stopProgress();
});
