/*
eslint-disable no-underscore-dangle
*/

import { ipcMain } from 'electron';

import ConnectionMonitor from '../../services/connectionMonitor';
/**
 * Browser connection monitor
 */
class BrowserConnectionMonitor extends ConnectionMonitor {
  constructor(params) {
    super({
      heartbeatInterval: 3000,
      heartbeatTimeout: 1500,
      ...params,
    });
    if (!params.win) {
      throw new Error('Browser connection monitor needs an instance of a browserWindow');
    }

    this._win = params.win;

    ipcMain.on('heartbeat pong', (event) => {
      if (this._win === event.sender.getOwnerBrowserWindow()) {
        this.messageReceived();
      }
    });
  }

  get serviceName() {
    return this.constructor.name;
  }

  _sendPing() {
    this._log('silly', 'Pinging browser window');

    this._win.webContents.send('heartbeat ping');
  }

  _logHeartbeatTimeout() {
    const url = this._win.webContents.getURL();
    this._log('error', `Heartbeat timeout reached for browser with URL: ${url}`, { serviceLog: true });
  }

  destroy() {
    super.destroy().catch(err => this._log('error', err.message));
    ipcMain.removeAllListeners(this._replyEventName);
    delete this._win;
  }
}

export default BrowserConnectionMonitor;
