export default function createQueue(passthrough) {
  let buffering = true;
  const buffer = [];
  return {
    get queued() {
      return buffering;
    },
    set queued(value) {
      buffering = value;
      // if setting queued to false, flush everything that was queued so far
      if (!buffering) {
        buffer.splice(0, buffer.length).forEach((args) => {
          passthrough(...args);
        });
      }
    },
    push(...args) {
      if (buffering) {
        buffer.push(args);
      } else {
        passthrough(...args);
      }
    },
    get length() {
      return buffer.length;
    },
  };
}
