import { expect } from 'chai';
import createQueue from './createQueue';

describe('utils/createQueue', () => {
  it('should start queued', () => {
    const queue = createQueue();
    expect(queue).to.haveOwnProperty('queued');
    expect(queue.queued).to.equal(true);
  });

  it('should not passthrough while queued', () => {
    const out = [];
    const queue = createQueue(d => out.push(d));

    queue.push('a');
    expect(out).to.be.empty;
  });

  it('should passthrough while not queued', () => {
    const out = [];
    const queue = createQueue(d => out.push(d));

    queue.push('a');
    queue.queued = false;
    expect(out).to.have.members(['a']);

    queue.push('b');
    expect(out).to.have.ordered.members(['a', 'b']);
  });

  it('should allow queueing to be resumed', () => {
    const out = [];
    const queue = createQueue(d => out.push(d));
    queue.queued = false;
    queue.push('a');
    expect(out).to.have.members(['a']);

    queue.queued = true;
    queue.push('b');
    expect(out).to.have.members(['a']);
  });
});
