import { EventEmitter2 } from 'eventemitter2';
/**
 * This class extends Node's event emitter, with capabilities to remove multiple listeners at once
 *  and it introduces the 'off' alias for removeListener, which is called by the destroy method
 */
class PurpleEventEmitter extends EventEmitter2 {
  constructor(options = {}) {
    super({
      wildcard: false,
      delimiter: ':',
      maxListeners: 500,
      ...options,
    });
  }

  /**
   * Alias for `removeListener`
   * @param event
   * @param listener
   * @returns {*}
   */
  off(event, listener) {
    return this.removeListener(event, listener);
  }

  /**
   * Combines removeListener and removeAllListeners in one method
   * @param event
   * @param listener
   * @returns {*|any|void}
   */
  removeListener(event, listener) {
    if (listener) {
      return super.removeListener(event, listener);
    }
    if (event) {
      return super.removeAllListeners(event);
    }
    return super.removeAllListeners();
  }

  /**
   * Destroys and cleans-up event emitter.
   * Removes all listeners
   */
  destroy() {
    this.off();
  }
}

export default PurpleEventEmitter;
