import * as fs from 'fs';
import * as path from 'path';
import * as JWT from 'jsonwebtoken';
import * as crypto from 'crypto';
import { promisify } from 'util';
import { getMacAddress } from './network';
import { RESOURCES_DIR, SEEN_JWT_FILE } from '../files';

const verifyToken = promisify(JWT.verify);
const key = fs.readFileSync(path.join(RESOURCES_DIR, 'jwt.pub'), 'utf-8');
let seen;
try {
  seen = fs.readFileSync(SEEN_JWT_FILE, 'utf-8').split('\n');
  seen.pop();
} catch (error) {
  seen = [];
}

function ensureUnique(token) {
  const hash = crypto.createHash('md5').update(token.trim()).digest('hex');
  if (seen.includes(hash)) {
    throw new Error('Token already used');
  } else {
    seen.push(hash);
    fs.appendFileSync(SEEN_JWT_FILE, `${hash}\n`);
  }
}

export default async function decodeToken(token, mustBeUnique = true) {
  const verifiedToken = await verifyToken(token, key, {
    algorithms: ['RS256'],
    audience: `weconnect-device-${getMacAddress()}`,
    issuer: 'weconnect-cloud',
  });

  if (mustBeUnique) {
    ensureUnique(token);
  }

  return verifiedToken;
}
