import { execSync } from 'child_process';

export default {
  get isWin() {
    return /^win/.test(process.platform);
  },
  setHWClock(date, log, syncSystemClock) {
    if (!(date instanceof Date)) {
      return;
    }
    log('info', 'Resetting HW clock to', date);
    try {
      execSync(`sudo hwclock --set --date="${date.toUTCString()}" --utc`);
      log('info', 'Reset HW clock to', date);
    } catch (error) {
      log('error', 'Error while resetting HW clock', error);
    }

    if (syncSystemClock) {
      log('info', 'Syncing system clock with hw clock');
      try {
        execSync('sudo timedatectl set-ntp off');
        execSync('sudo hwclock --hctosys');
        execSync('sudo timedatectl set-ntp on');
        log('info', 'System clock synced');
      } catch (error) {
        log('error', 'Error while syncing system clock', error);
      }
    }
  },
};
