import { ipcMain } from 'electron';
import SpeechToTextStream from './SpeechToTextStream';
import Random from '../random';
import channelId from './channelId';
import { IPC_SPEECH_TO_TEXT } from '../../ipc-constants';

const instances = {};

function destroyInstance(id) {
  if (instances[id]) {
    const instance = instances[id];
    return instance.destroy();
  }
}

ipcMain.handle(IPC_SPEECH_TO_TEXT.CREATE_SPEECH_TO_TEXT, async (event, {
  clientConfig,
  recordingConfig,
  sessionConfig,
  type,
} = {}) => {
  const id = Random.id();
  const instance = new SpeechToTextStream(id, clientConfig, recordingConfig, sessionConfig, type);
  instance.on('result', (message) => {
    event.sender.send(channelId(id), message);
  });
  instance.on('destroyed', () => {
    delete instances[id];
  });
  event.sender.on('destroyed', () => {
    destroyInstance(id);
  });
  instances[id] = instance;
  instance.start();
  return id;
});

ipcMain.handle(IPC_SPEECH_TO_TEXT.STOP_SPEECH_TO_TEXT, (event, id) => destroyInstance(id));
