import serviceRegistry from '../../services/serviceRegistry';
import Logger from '../../services/logger';
import Messenger from '../../services/messenger';

class FakeLogger extends Logger {
  constructor(options) {
    options.dir = 'tmp';
    options.maxFiles = 0;
    super(options);
  }

  getInstance() {
    return this;
  }

  _log() {}

  log() {}

  removeListener() {}
}

class FakeMessenger {
  start() {}

  on() {}

  show() {
    return Promise.resolve({
      hide() {
        return true;
      },
    });
  }

  removeListener() {}
}

const reals = {
  logger: Logger,
  messenger: Messenger,
};

const fakes = {
  logger: FakeLogger,
  messenger: FakeMessenger,
};

function registerLogger(fake = true) {
  const from = fake ? fakes : reals;
  const loggerService = new from.logger({
    //eslint-disable-line
    dir: 'tmp',
    filename: 'test.log',
    level: 'debug',
    maxSize: 5242880,
    maxFiles: 5, // only log to console
  });

  serviceRegistry.set('logger', loggerService);
}

function registerMessenger(fake = true) {
  const messenger = fake ? new FakeMessenger() : new Messenger();
  messenger.start();
  serviceRegistry.set('messenger', messenger);
}

function registerAll(fake = true) {
  registerLogger(fake);
  registerMessenger(fake);
}

function unregisterAll() {
  serviceRegistry.delete('logger');
  serviceRegistry.delete('messenger');
}

export {
  registerAll, unregisterAll, registerLogger, registerMessenger,
};
