import * as fs from 'fs-extra';

import { VERSION_FILE, NRC_VERSION_FILE } from '../files';
import { copySafely } from './fs';

function readCurrentVersion(logger, readNRCVersion = false) {
  const versionFile = readNRCVersion ? NRC_VERSION_FILE : VERSION_FILE;
  const versionString = readNRCVersion ? 'NRC' : 'system';
  try {
    logger.log('info', `Fetching current ${versionString} version`);
    let currentVersion = fs.readFileSync(versionFile, 'utf-8');
    currentVersion = currentVersion.replace(/(\r\n|\n|\r)/gm, '');
    logger.log('info', `Current ${versionString} version: ${currentVersion}`);
    return currentVersion;
  } catch (error) {
    logger.log('error', `Error while fetching current ${versionString} version`, error);
  }
}

function copyVersionFile(logger) {
  try {
    logger.log('info', 'Copying version file');
    copySafely(NRC_VERSION_FILE, VERSION_FILE);
  } catch (error) {
    logger.log('error', 'Error while copying version file', error);
  }
}

/**
   * On a clean purple installation, there is no version file
   * In this situation, we copy the nrc version file
   */
function ensureVersionFile(logger) {
  // Purple keeps its own version file to avoid failed upgrades
  // E.g. the scenario where the upgrade fails/crashes,
  // but the cinergy package has already updated, and thus the nrc_version file.
  // Purple will only update the version file after a successfull upgrade
  // even if the device crashes mid-upgrade, it will retry the upgrade this way.
  logger.log('info', 'Checking if version file exists');
  const versionFileExists = fs.existsSync(VERSION_FILE);
  if (versionFileExists) {
    logger.log('info', 'Version file exists, continuing');
  } else {
    logger.log('info', `Version file does not exist => copy from ${NRC_VERSION_FILE}`);
    copyVersionFile(logger);
  }
}

export {
  readCurrentVersion,
  ensureVersionFile,
  copyVersionFile,
};
