/*
eslint-disable
*/
let xrandrOutput1 = `Screen 0: minimum 320 x 200, current 3926 x 1440, maximum 8192 x 8192
XWAYLAND1 connected 1366x768+0+567 (0x25) normal (normal left inverted right x axis y axis) 280mm x 160mm
  Identifier: 0x23
  Timestamp:  167562
  Subpixel:   unknown
  Gamma:      1.0:1.0:1.0
  Brightness: 0.0
  Clones:
  CRTC:       0
  CRTCs:      0
  Transform:  1.000000 0.000000 0.000000
              0.000000 1.000000 0.000000
              0.000000 0.000000 1.000000
             filter:
  1366x768 (0x25) 84.750MHz -HSync +VSync *current +preferred
        h: width  1366 start 1431 end 1567 total 1776 skew    0 clock  47.72KHz
        v: height  768 start  771 end  781 total  798           clock  59.80Hz
XWAYLAND46 connected 2560x1440+1366+0 (0x24) normal (normal left inverted right x axis y axis) 620mm x 340mm
  Identifier: 0x2fa
  Timestamp:  167562
  Subpixel:   unknown
  Gamma:      1.0:1.0:1.0
  Brightness: 0.0
  Clones:
  CRTC:       1
  CRTCs:      1
  Transform:  1.000000 0.000000 0.000000
              0.000000 1.000000 0.000000
              0.000000 0.000000 1.000000
             filter:
  2560x1440 (0x24) 312.000MHz -HSync +VSync *current +preferred
        h: width  2560 start 2752 end 3024 total 3488 skew    0 clock  89.45KHz
        v: height 1440 start 1443 end 1448 total 1493           clock  59.91Hz`;

function  _transformXrandrOutput(src, {
    includeVirtual = false,
    includeDisconnected = false,
    includeEdid = false,
    includeInterlaced = false,
    includeDimensions = false,
  } = {}) {
    const type = {
      connected: /^(\S+) connected(?: primary)? (?:(\d+)x(\d+)\+(\d+)\+(\d))*\s*(\(\w+\))*\s*(\w*)/,
      edid: /\tEDID:/,
      edid_block: /^\t\t([\da-f]{32})$/i,
      disconnected: /^(\S+) disconnected/,
      mode: /^\s+(\d+)x([0-9i]+)\s+(\(\w+\))\s+([0-9]+\.[0-9]+)MHz\s+((\s*[+]*[-]*\w*\s)*)/,
      dimension_horizontal: /^\s+h:\s+width\s+(\d+)\s+start\s+(\d+)\s+end\s+(\d+)\s+total\s+(\d+)\s*skew\s*(\w)\s*clock\s*([0-9]*.[0-9]*)/,
      dimension_vertical: /^\s+v:\s+height\s+(\d+)\s+start\s+(\d+)\s+end\s+(\d+)\s+total\s+(\d+)\s*clock\s*([0-9]*.[0-9]*)/,
    };

    const lines = src.split('\n');
    let result = [];
    let currentConnection = null;
    let currentMode = null;
    let index = 0;

    let processingEdid = false;
    let edidBlock = '';

    lines.forEach((line) => {
      let tmp;
      if ((tmp = type.connected.exec(line))) {
        let [_1, port, width, height, left, top, _7, orientation] = tmp;
        width = width || 0;
        height = height || 0;
        left = left || 0;
        top = top || 0;
        orientation = orientation || 'normal';
        index += 1;
        const connection = {
          port,
          connected: true,
          orientation,
          modes: [],
          index,
        };

        connection.width = Number.parseInt(width, 10);
        connection.height = Number.parseInt(height, 10);
        connection.left = Number.parseInt(left, 10);
        connection.top = Number.parseInt(top, 10);

        currentConnection = connection;
        result.push(currentConnection);
      } else if ((tmp = type.disconnected.exec(line))) {
        const [_0, port] = tmp;
        index += 1;
        currentConnection = {
          port,
          connected: false,
          modes: [],
          index,
        };

        result.push(currentConnection);
      } else if ((tmp = type.mode.exec(line))) {
        const [_0, width, height, _3, rate, optionals] = tmp;

        const mode = {
          name: `${width}x${height}`,
          width: Number.parseInt(width, 10),
          height: Number.parseInt(height, 10),
          rate: parseFloat(rate),
          optionals,
          current: !!line.includes('current'),
          native: !!line.includes('preferred'),
        };

        if (includeDimensions) {
          mode.dimensions = {
            vertical: null,
            horizontal: null,
          };
        }

        mode.interlaced = mode.name.includes('i');

        currentConnection.modes.push(mode);
        currentMode = mode;
      } else if (includeDimensions && (tmp = type.dimension_horizontal.exec(line))) {
        const [_, width, start, end, total, skew, clock] = tmp;
        const dimensionH = {
          width: Number.parseInt(width, 10),
          start: Number.parseInt(start, 10),
          end: Number.parseInt(end, 10),
          total: Number.parseInt(total, 10),
          skew: Number.parseInt(skew, 10),
          clock: parseFloat(clock),
        };
        currentMode.dimensions.horizontal = dimensionH;
      } else if ((tmp = type.dimension_vertical.exec(line))) {
        const [_, width, start, end, total, clock] = tmp;
        const dimensionV = {
          width: Number.parseInt(width, 10),
          start: Number.parseInt(start, 10),
          end: Number.parseInt(end, 10),
          total: Number.parseInt(total, 10),
          clock: parseFloat(clock),
        };

        currentMode.rate = dimensionV.clock;

        if (includeDimensions) {
          currentMode.dimensions.vertical = dimensionV;
        }
      } else if (includeEdid && (tmp = type.edid.exec(line))) {
        processingEdid = true;
        edidBlock = '';
      } else if (processingEdid) {
        if ((tmp = type.edid_block.exec(line))) {
          edidBlock += tmp[0].trim();
        } else {
          processingEdid = false;
          currentConnection.edid = EdidReader.parse(edidReader.formatEdid(edidBlock));
          edidBlock = '';
        }
      }
    });

    if (!includeDisconnected) {
      result = result.filter(d => d.connected);
    }

    if (!includeVirtual) {
      result = result.filter(d => !d.port.includes('VIRTUAL'));
    }

    if (!includeInterlaced) {
      result.forEach((d) => {
        d.modes = d.modes.filter(m => !m.interlaced);
      });
    }

    return result;
  }

const output= _transformXrandrOutput(xrandrOutput1);
debugger;

